@@ -12,7 +12,7 @@ use warnings;
 
 no warnings qw( threads recursion uninitialized once redefine );
 
-our $VERSION = '1.808';
+our $VERSION = '1.809';
 
 ## no critic (BuiltinFunctions::ProhibitStringyEval)
 ## no critic (Subroutines::ProhibitExplicitReturnUndef)
@@ -123,7 +123,7 @@ sub create {
 
    bless $self, $class;
 
-   ## error checking and setup  --- --- --- --- --- --- --- --- --- --- --- ---
+   ## error checks and setup -- --- --- --- --- --- --- --- --- --- --- --- ---
 
    if ( ref($func) ne 'CODE' && !length($func) ) {
       return $self->_error(
@@ -137,6 +137,7 @@ sub create {
       $self->{posix_exit} = 1 if ( $_has_threads && $_tid );
       $self->{posix_exit} = 1 if ( $INC{'CGI.pm'} || $INC{'FCGI.pm'} );
       $self->{posix_exit} = 1 if ( $INC{'Mojo/IOLoop.pm'} || $INC{'Tk.pm'} );
+      $self->{posix_exit} = 1 if ( $INC{'Gearman/XS.pm'} );
    }
 
    if ( !exists $_LIST->{$pkg} ) {
@@ -158,12 +159,12 @@ sub create {
       $_LIST->{$pkg}->clear();
    }
 
-   ## spawn a hobo process  --- --- --- --- --- --- --- --- --- --- --- --- ---
+   ## spawn hobo worker --- --- --- --- --- --- --- --- --- --- --- --- --- ---
 
    my $seed = $_STAT->{$pkg}->get("$mgr_id:seed");
    my $id   = $_STAT->{$pkg}->incr("$mgr_id:id");
 
-   ( $_is_MSWin32 )
+   $_is_MSWin32
       ? _dispatch_thread ( $self, $pkg, $seed, $id, $mgr_id, $func, @_ )
       : _dispatch_child  ( $self, $pkg, $seed, $id, $mgr_id, $func, @_ );
 }
@@ -203,7 +204,7 @@ sub exit {
       if ( $_is_MSWin32 ) {
          sleep(0.015), $self->{_thr}->kill('QUIT');
       } else {
-         sleep(0.015), CORE::kill('QUIT', $wrk_id);
+         sleep(0.015), CORE::kill('QUIT', $wrk_id) if CORE::kill('ZERO', $wrk_id);
       }
 
       $self;
@@ -354,7 +355,7 @@ sub kill {
    my $pkg    = caller;
 
    if ( $wrk_id == ( $_is_MSWin32 ? $_tid : $$ ) ) {
-      ( $_is_MSWin32 )
+      $_is_MSWin32
          ? threads->self->kill($signal || 'INT')
          : CORE::kill($signal || 'INT', $$);
    }
@@ -365,13 +366,17 @@ sub kill {
       if ( $_is_MSWin32 ) {
          sleep(0.015), $self->{_thr}->kill($signal || 'INT');
       } else {
-         sleep(0.015), CORE::kill($signal || 'INT', $wrk_id);
+         sleep(0.015), CORE::kill($signal || 'INT', $wrk_id)
+            if CORE::kill('ZERO', $wrk_id);
       }
    }
    else {
-      ( $_is_MSWin32 )
-         ? $self->{_thr}->kill($signal || 'INT')
-         : CORE::kill($signal || 'INT', $wrk_id);
+      if ( $_is_MSWin32 ) {
+         $self->{_thr}->kill($signal || 'INT');
+      } else {
+         CORE::kill($signal || 'INT', $wrk_id)
+            if CORE::kill('ZERO', $wrk_id);
+      }
    }
 
    $self;
@@ -557,14 +562,20 @@ sub _dispatch {
 
    ## Run.
 
+   local $SIG{'ALRM'} = sub { alarm 0; die "Hobo timed out\n" };
    $_STAT->{$pkg}->get($mgr_id)->set($wrk_id, '');
-   my @res = eval { no strict 'refs'; $func->(@args) };
 
-   delete $_SELF->{_pkg};
+   my @res = eval {
+      alarm( $self->{hobo_timeout} || 0 );
+      no strict 'refs'; $func->(@args);
+   };
+
+   alarm 0; delete $_SELF->{_pkg};
 
    if ( $@ && $@ ne "Hobo exited (0)\n" ) {
       $_STAT->{$pkg}->get($mgr_id)->set($wrk_id, $@);
-      warn "Hobo $wrk_id terminated abnormally: $@\n";
+      warn "Hobo $wrk_id terminated abnormally: reason $@\n"
+         if ( $@ ne "Hobo timed out\n" );
    }
 
    $_DATA->{$pkg}->get($mgr_id)->set($wrk_id, $_freeze->(\@res));
@@ -575,8 +586,7 @@ sub _dispatch {
 sub _dispatch_child {
    my ( $self, $pkg, $seed, $id, $mgr_id, $func, @args ) = @_;
 
-   @_ = (); local $_ = $_;
-   my $pid = fork();
+   @_ = (); local $_ = $_; my $pid = fork();
 
    return $self->_error("fork error: $!\n") if (!defined $pid);
 
@@ -622,13 +632,9 @@ sub _exit {
    $SIG{__DIE__}  = sub { } unless $_tid;
    $SIG{__WARN__} = sub { };
 
-   if ( $_is_MSWin32 ) {
-      threads->exit(0);
-   }
-   elsif ( $_SELF->{posix_exit} ) {
-      CORE::kill('KILL', $$);
-   }
+   threads->exit(0) if $_is_MSWin32;
 
+   CORE::kill('KILL', $$) if $_SELF->{posix_exit};
    CORE::exit(0);
 }
 
@@ -639,7 +645,7 @@ sub _notify {
 
    if ( $? ) {
       $_STAT->{$pkg}->get($mgr_id)->set($wrk_id, "Hobo exited ($?)\n");
-      warn "Hobo $wrk_id terminated abnormally: Hobo exited ($?)\n\n";
+      warn "Hobo $wrk_id terminated abnormally: reason Hobo exited ($?)\n\n";
    }
 
    $_DATA->{$pkg}->get($mgr_id)->set($wrk_id, $_freeze->([]));
@@ -668,7 +674,7 @@ MCE::Hobo - A threads-like parallelization module
 
 =head1 VERSION
 
-This document describes MCE::Hobo version 1.808
+This document describes MCE::Hobo version 1.809
 
 =head1 SYNOPSIS
 
@@ -700,7 +706,7 @@ This document describes MCE::Hobo version 1.808
    $hobo->join();
 
    if ( my $err = $hobo->error() ) {
-      warn("Hobo error: $err\n");
+      warn "Hobo error: $err\n";
    }
 
    # Get a hobo's object
@@ -748,10 +754,9 @@ process to the underlying OS. The IPC is managed by C<MCE::Shared>,
 which runs on all the major platforms including Cygwin.
 
 C<MCE::Hobo> may be used as a standalone or together with C<MCE>
-including running alongside C<threads>.
-
-Beginning with C<MCE::Shared> 1.807, an exception is made on the Windows
-platform to use threads instead of fork for better reliability.
+including running alongside C<threads>. An exception is made on the
+Windows platform in C<MCE::Shared> 1.807 and later to spawn workers
+via threads instead of child processes for better reliability.
 
 The following is a parallel demonstration.
 
@@ -794,8 +799,6 @@ The following is a parallel demonstration.
 
 =over 3
 
-=item $hobo = MCE::Hobo->create( { posix_exit => 1 }, FUNCTION, ARGS )
-
 =item $hobo = MCE::Hobo->create( FUNCTION, ARGS )
 
 =item $hobo = MCE::Hobo->new( FUNCTION, ARGS )
@@ -804,10 +807,6 @@ This will create a new hobo that will begin execution with function as the
 entry point, and optionally ARGS for list of parameters. It will return the
 corresponding MCE::Hobo object, or undef if hobo creation failed.
 
-Options may be specified via a hash structure. At this time, C<posix_exit> is
-the only option supported. Set C<posix_exit> to avoid all END and destructor
-processing. Constructing a Hobo inside a thread implies C<posix_exit>.
-
 I<FUNCTION> may either be the name of a function, an anonymous subroutine, or
 a code ref.
 
@@ -817,6 +816,33 @@ a code ref.
        # or
    my $hobo = MCE::Hobo->create( \&func, ... );
 
+=item $hobo = MCE::Hobo->create( { options }, FUNCTION, ARGS )
+
+Options may be specified via a hash structure. At this time, C<posix_exit> and
+C<hobo_timeout> are the only options supported. Set C<posix_exit> to avoid all
+END and destructor processing. Set C<hobo_timeout>, in number of seconds, if
+you want the hobo process to terminate after some time. The default is C<0>
+for no timeout.
+
+Constructing a Hobo inside a thread implies C<posix_exit => 1> or if present
+CGI.pm, FCGI.pm, Gearman::XS, Mojo::IOLoop, or Tk.
+
+   my $hobo1 = MCE::Hobo->create( { posix_exit => 1 }, sub {
+      ...
+   } );
+
+   $hobo1->join;
+
+   my $hobo2 = MCE::Hobo->create( { hobo_timeout => 3 }, sub {
+      sleep 1 for ( 1 .. 9 );
+   } );
+
+   $hobo2->join;
+
+   if ( $hobo2->error() eq "Hobo timed out\n" ) {
+      ...
+   }
+
 The C<new()> method is an alias for C<create()>.
 
 =item mce_async { BLOCK } ARGS;
@@ -1079,11 +1105,11 @@ Class method that allows a hobo to obtain it's own I<MCE::Hobo> object.
 
 Returns the ID of the hobo.
 
-   PID: $$.$tid  # Windows (only), includes thread id
-   TID: $$.$tid  # same thing
+   pid: $$.$tid  # Windows (only), with thread id
+   tid: $$.$tid  # same thing
 
-   PID: $$       # Cygwin and UNIX platforms
-   TID: $$       # same thing
+   pid: $$       # Cygwin and UNIX platforms
+   tid: $$
 
 =item MCE::Hobo->pid()
 
@@ -1091,6 +1117,12 @@ Returns the ID of the hobo.
 
 Class methods that allows a hobo to obtain its own ID.
 
+   pid: $$.$tid  # Windows (only), with thread id
+   tid: $$.$tid  # same thing
+
+   pid: $$       # Cygwin and UNIX platforms
+   tid: $$
+
 =item MCE::Hobo->waitone()
 
 =item MCE::Hobo->waitall()
@@ -1147,6 +1179,47 @@ C<sleep(0.0005)> on UNIX and C<sleep(0.015)> on Windows including Cygwin.
 
 =back
 
+=head1 CROSS-PLATFORM TEMPLATE FOR BINARY EXECUTABLE
+
+Making an executable is possible with the L<PAR::Packer> module.
+On the Windows platform, threads, threads::shared, and exiting via
+threads are all necessary for the binary to exit successfully.
+
+   # https://metacpan.org/pod/PAR::Packer
+   # https://metacpan.org/pod/pp
+   #
+   #   pp -o demo.exe demo.pl
+   #   ./demo.exe
+
+   use strict;
+   use warnings;
+
+   use if $^O eq "MSWin32", "threads";
+   use if $^O eq "MSWin32", "threads::shared";
+
+   use Time::HiRes (); # include minimum dependencies for MCE::Hobo
+   use Storable ();
+
+   use IO::FDPass ();  # optional: for MCE::Shared->condvar, handle, queue
+   use Sereal ();      # optional: faster serialization, may omit Storable
+
+   use MCE::Hobo;      # 1.808 or later on Windows
+   use MCE::Shared;
+
+   my $seq_a = MCE::Shared->sequence( 1, 30 );
+
+   sub task {
+      my ( $id ) = @_;
+      while ( defined ( my $num = $seq_a->next ) ) {
+         print "$id: $num\n";
+      }
+   }
+
+   MCE::Hobo->new( \&task, $_ ) for 1 .. 2;
+   MCE::Hobo->waitall;
+
+   threads->exit(0) if $INC{"threads.pm"};
+
 =head1 CREDITS
 
 The inspiration for C<MCE::Hobo> comes from wanting C<threads>-like behavior
