@@ -12,7 +12,7 @@ use warnings;
 
 no warnings qw( threads recursion uninitialized once );
 
-our $VERSION = '1.808';
+our $VERSION = '1.809';
 
 ## no critic (BuiltinFunctions::ProhibitStringyEval)
 ## no critic (Subroutines::ProhibitSubroutinePrototypes)
@@ -25,7 +25,7 @@ our @CARP_NOT = qw(
    MCE::Shared::Array   MCE::Shared::Condvar  MCE::Shared::Handle
    MCE::Shared::Hash    MCE::Shared::Minidb   MCE::Shared::Ordhash
    MCE::Shared::Queue   MCE::Shared::Scalar   MCE::Shared::Sequence
-   MCE::Shared::Server  MCE::Shared::Object
+   MCE::Shared::Server  MCE::Shared::Object   MCE::Shared::Cache
 );
 
 sub import {
@@ -119,30 +119,35 @@ sub init {
    MCE::Shared::Object::_init(@_);
 }
 
+sub cache {
+   shift if ( defined $_[0] && $_[0] eq 'MCE::Shared' );
+   require MCE::Shared::Cache unless $INC{'MCE/Shared/Cache.pm'};
+   &share({}, MCE::Shared::Cache->new(@_));
+}
 sub condvar {
    shift if ( defined $_[0] && $_[0] eq 'MCE::Shared' );
    require MCE::Shared::Condvar unless $INC{'MCE/Shared/Condvar.pm'};
-   &share( MCE::Shared::Condvar->new(@_) );
+   &share({}, MCE::Shared::Condvar->new(@_));
 }
 sub minidb {
    shift if ( defined $_[0] && $_[0] eq 'MCE::Shared' );
    require MCE::Shared::Minidb unless $INC{'MCE/Shared/Minidb.pm'};
-   &share( MCE::Shared::Minidb->new(@_) );
+   &share({}, MCE::Shared::Minidb->new(@_));
 }
 sub queue {
    shift if ( defined $_[0] && $_[0] eq 'MCE::Shared' );
    require MCE::Shared::Queue unless $INC{'MCE/Shared/Queue.pm'};
-   &share( MCE::Shared::Queue->new(@_) );
+   &share({}, MCE::Shared::Queue->new(@_));
 }
 sub scalar {
    shift if ( defined $_[0] && $_[0] eq 'MCE::Shared' );
    require MCE::Shared::Scalar unless $INC{'MCE/Shared/Scalar.pm'};
-   &share( MCE::Shared::Scalar->new(@_) );
+   &share({}, MCE::Shared::Scalar->new(@_));
 }
 sub sequence {
    shift if ( defined $_[0] && $_[0] eq 'MCE::Shared' );
    require MCE::Shared::Sequence unless $INC{'MCE/Shared/Sequence.pm'};
-   &share( MCE::Shared::Sequence->new(@_) );
+   &share({}, MCE::Shared::Sequence->new(@_));
 }
 
 ## 'num_sequence' is an alias for 'sequence'
@@ -271,20 +276,30 @@ sub TIESCALAR { shift; MCE::Shared->scalar(@_) }
 
 sub TIEHASH {
    shift;
-   my $_ordered;
+   my ($_cache, $_ordered);
 
    if ( ref $_[0] eq 'HASH' ) {
-      shift(), $_ordered = 1 if ( $_[0]->{'ordered'} || $_[0]->{'ordhash'} );
+      if ( $_[0]->{'ordered'} || $_[0]->{'ordhash'} ) {
+         $_ordered = 1; shift();
+      } elsif ( exists $_[0]->{'max_age'} || exists $_[0]->{'max_keys'} ) {
+         $_cache   = 1;
+      }
    }
    else {
-      shift(), shift(), $_ordered = 1 if (
-         @_ < 3 && ( $_[0] eq 'ordered' || $_[0] eq 'ordhash' ) && $_[1]
-      );
+      if ( @_ < 3 && ( $_[0] eq 'ordered' || $_[0] eq 'ordhash' ) ) {
+         $_ordered = $_[1]; splice(@_, 0, 2);
+      } elsif ( @_ < 5 && ( $_[0] eq 'max_age' || $_[0] eq 'max_keys' ) ) {
+         $_cache   = 1;
+      }
    }
 
-   ( $_ordered )
-      ? MCE::Shared->ordhash(@_)
-      : MCE::Shared->hash(@_);
+   if ( $_cache ) {
+      MCE::Shared->cache(@_);
+   } elsif ( $_ordered ) {
+      MCE::Shared->ordhash(@_);
+   } else {
+      MCE::Shared->hash(@_);
+   }
 }
 
 sub TIEHANDLE {
@@ -342,7 +357,7 @@ MCE::Shared - MCE extension for sharing data supporting threads and processes
 
 =head1 VERSION
 
-This document describes MCE::Shared version 1.808
+This document describes MCE::Shared version 1.809
 
 =head1 SYNOPSIS
 
@@ -351,6 +366,7 @@ This document describes MCE::Shared version 1.808
    use MCE::Shared;
 
    my $ar = MCE::Shared->array( @list );
+   my $ca = MCE::Shared->cache( max_keys => 500, max_age => 60 );
    my $cv = MCE::Shared->condvar( 0 );
    my $fh = MCE::Shared->handle( '>>', \*STDOUT );
    my $ha = MCE::Shared->hash( @pairs );
@@ -361,9 +377,9 @@ This document describes MCE::Shared version 1.808
    my $se = MCE::Shared->sequence( $begin, $end, $step, $fmt );
    my $ob = MCE::Shared->share( $blessed_object );
 
-   # open function, MCE::Shared 1.002 and later
+   # open function in MCE::Shared 1.002 and later
 
-   mce_open my $fh, ">", "/foo/bar.log" or die "$!";
+   mce_open my $fh, ">", "/foo/bar.log" or die "open error: $!";
 
    # Tie construction
 
@@ -376,6 +392,7 @@ This document describes MCE::Shared version 1.808
    tie my @ary, 'MCE::Shared', qw( a list of values );
    tie my %ha,  'MCE::Shared', ( key1 => 'value', key2 => 'value' );
    tie my %oh,  'MCE::Shared', { ordered => 1 }, ( key1 => 'value' );
+   tie my %ca,  'MCE::Shared', { max_keys => 500, max_age => 60 };
 
    tie my $cnt, 'MCE::Shared', 0;
    tie my @foo, 'MCE::Shared';
@@ -427,45 +444,41 @@ This module provides data sharing capabilities for L<MCE> supporting threads
 and processes. L<MCE::Hobo> provides threads-like parallelization for running
 code asynchronously.
 
-C<MCE::Shared> enables extra functionality on systems with C<IO::FDPass>
+C<MCE::Shared> enables extra functionality on systems with L<IO::FDPass>
 installed. Without it, MCE::Shared is unable to send file descriptors to the
-shared-manager process for C<queue>, C<condvar>, and possibly C<handle>.
-
-As of this writing, the L<IO::FDPass> module is not a requirement for running
-C<MCE::Shared> nor is the check made during installation. The reason is that
-C<IO::FDPass> is not possible on Cygwin and not sure about AIX.
-
-The following is a suggestion for systems without C<IO::FDPass>.
-This restriction applies to C<queue>, C<condvar>, and C<handle> only.
+shared-manager process. The following is a suggestion for systems without
+C<IO::FDPass>. This restriction applies to sharing L<MCE::Shared::Condvar>,
+L<MCE::Shared::Handle>, and L<MCE::Shared::Queue> only.
 
    use MCE::Shared;
 
-   # Construct any shared queue(s) and condvar(s) first.
+   # Construct any shared Condvar(s), Handle(s), and Queue(s) first.
    # These contain GLOB handles where freezing is not allowed.
 
-   my $q1  = MCE::Shared->queue();
-   my $q2  = MCE::Shared->queue();
-
    my $cv1 = MCE::Shared->condvar();
    my $cv2 = MCE::Shared->condvar();
 
-   # Afterwards, start the shared-manager manually.
+   my $q1  = MCE::Shared->queue();
+   my $q2  = MCE::Shared->queue();
 
-   MCE::Shared->start();
-
-   # The shared-manager process knows of \*STDOUT, \*STDERR, \*STDIN
+   # The shared-manager process knows of ( \*STDOUT, \*STDERR, \*STDIN ).
+   # Therefore okay to construct these after the manager is running.
 
    mce_open my $fh1, ">>", \*STDOUT;                  # ok
    mce_open my $fh2, "<", "/path/to/sequence.fasta";  # ok
-   my $h1 = MCE::Shared->hash();                      # ok
 
-Otherwise, sharing is immediate and not delayed with C<IO::FDPass>. It is not
-necessary to share C<queue> and C<condvar> first or worry about starting the
-shared-manager process.
+   # Afterwards, start the shared-manager manually.
+   # Not necessary if IO::FDPass is available.
+
+   MCE::Shared->start();
+
+The shared-manager starts automatically for other classes shipped with
+C<MCE::Shared>. Therefore, the following will fail if Perl lacks the
+C<IO::FDPass> module.
 
    use MCE::Shared;
 
-   my $h1 = MCE::Shared->hash();    # shares immediately
+   my $h1 = MCE::Shared->hash();    # shared-manager starts immediately
    my $q1 = MCE::Shared->queue();   # IO::FDPass sends file descriptors
    my $cv = MCE::Shared->condvar(); # IO::FDPass sends file descriptors
    my $h2 = MCE::Shared->ordhash();
@@ -474,60 +487,71 @@ shared-manager process.
 
 =over 3
 
-=item * array
+=item * array L<MCE::Shared::Array>
 
-=item * condvar
+=item * cache L<MCE::Shared::Cache>
 
-=item * handle
+=item * condvar L<MCE::Shared::Condvar>
 
-=item * hash
+=item * handle L<MCE::Shared::Handle>
 
-=item * minidb
+=item * hash L<MCE::Shared::Hash>
 
-=item * ordhash
+=item * minidb L<MCE::Shared::Minidb>
 
-=item * queue
+=item * ordhash L<MCE::Shared::Ordhash>
 
-=item * scalar
+=item * queue L<MCE::Shared::Queue>
 
-=item * sequence
+=item * scalar L<MCE::Shared::Scalar>
+
+=item * sequence L<MCE::Shared::Sequence>
 
 =back
 
-C<array>, C<condvar>, C<handle>, C<hash>, C<minidb>, C<ordhash>, C<queue>,
-C<scalar>, and C<sequence> are sugar syntax for constructing a shared object.
-
-   # long form
-
-   use MCE::Shared;
-
-   use MCE::Shared::Array;
-   use MCE::Shared::Hash;
-   use MCE::Shared::OrdHash;
-   use MCE::Shared::Minidb;
-   use MCE::Shared::Queue;
-   use MCE::Shared::Scalar;
-
-   my $ar = MCE::Shared->share( MCE::Shared::Array->new() );
-   my $ha = MCE::Shared->share( MCE::Shared::Hash->new() );
-   my $oh = MCE::Shared->share( MCE::Shared::Ordhash->new() );
-   my $db = MCE::Shared->share( MCE::Shared::Minidb->new() );
-   my $qu = MCE::Shared->share( MCE::Shared::Queue->new() );
-   my $va = MCE::Shared->share( MCE::Shared::Scalar->new() );
+Below, synopsis for sharing classes included with MCE::Shared.
 
    # short form
 
    use MCE::Shared;
 
-   my $ar = MCE::Shared->array( @list );
-   my $cv = MCE::Shared->condvar( 0 );
-   my $fh = MCE::Shared->handle( '>>', \*STDOUT );
-   my $ha = MCE::Shared->hash( @pairs );
-   my $oh = MCE::Shared->ordhash( @pairs );
-   my $db = MCE::Shared->minidb();
-   my $qu = MCE::Shared->queue( await => 1, fast => 0 );
-   my $va = MCE::Shared->scalar( $value );
-   my $se = MCE::Shared->sequence( $begin, $end, $step, $fmt );
+   $ar = MCE::Shared->array( @list );
+   $ca = MCE::Shared->cache( max_keys => 500, max_age => 60 );
+   $cv = MCE::Shared->condvar( 0 );
+   $fh = MCE::Shared->handle( ">>", \*STDOUT ); # see mce_open below
+   $ha = MCE::Shared->hash( @pairs );
+   $oh = MCE::Shared->ordhash( @pairs );
+   $db = MCE::Shared->minidb();
+   $qu = MCE::Shared->queue( await => 1, fast => 0 );
+   $va = MCE::Shared->scalar( $value );
+   $se = MCE::Shared->sequence( $begin, $end, $step, $fmt );
+
+   mce_open my $fh, ">>", \*STDOUT or die "open error: $!";
+
+   # long form, must include class module
+
+   use MCE::Shared::Array;
+   use MCE::Shared::Cache;
+   use MCE::Shared::Hash;
+   use MCE::Shared::Minidb;
+   use MCE::Shared::Ordhash;
+   use MCE::Shared::Queue;
+   use MCE::Shared::Scalar;
+
+   $ar = MCE::Shared->share( MCE::Shared::Array->new( ... ) );
+   $ca = MCE::Shared->share( MCE::Shared::Cache->new( ... ) );
+   $ha = MCE::Shared->share( MCE::Shared::Hash->new( ... ) );
+   $db = MCE::Shared->share( MCE::Shared::Minidb->new( ... ) );
+   $oh = MCE::Shared->share( MCE::Shared::Ordhash->new( ... ) );
+   $qu = MCE::Shared->share( MCE::Shared::Queue->new( ... ) );
+   $va = MCE::Shared->share( MCE::Shared::Scalar->new( ... ) );
+
+The restriction for sharing classes not included with MCE::Shared
+is that the object must not have file-handles nor code-blocks.
+
+   use Hash::Ordered;
+
+   $oh = MCE::Shared->share( Hash::Ordered->new( ... ) );
 
 =over 3
 
@@ -544,23 +568,29 @@ a message and stop if open fails.
 
 See L<MCE::Shared::Handle> for chunk IO demonstrations.
 
-   # non-shared
+   # non-shared or local construction for use by a single process
+
    use MCE::Shared::Handle;
 
    MCE::Shared::Handle->open( my $fh, "<", "file.log" ) or die "$!";
    MCE::Shared::Handle::open  my $fh, "<", "file.log"   or die "$!";
 
-   # shared
+   mce_open my $fh, "<", "file.log" or die "$!";
+
+   # construction for sharing with other threads and processes
+
    use MCE::Shared;
 
    MCE::Shared->open( my $fh, "<", "file.log" ) or die "$!";
    MCE::Shared::open  my $fh, "<", "file.log"   or die "$!";
 
+   mce_open my $fh, "<", "file.log" or die "$!";
+
 Simple examples to open a file for reading:
 
-   # mce_open, exported
-   # creates a shared handle when MCE::Shared is present
-   # creates a non-shared handle, otherwise
+   # mce_open is exported by MCE::Shared or MCE::Shared::Handle.
+   # It creates a shared file handle with MCE::Shared present
+   # or a non-shared handle otherwise.
 
    mce_open my $fh, "< input.txt"     or die "open error: $!";
    mce_open my $fh, "<", "input.txt"  or die "open error: $!";
@@ -689,8 +719,8 @@ For further reading, see L<MCE::Shared::Minidb>.
 
 This class method transfers the blessed-object to the shared-manager
 process and returns a C<MCE::Shared::Object> containing the C<SHARED_ID>.
-The object must not contain any C<GLOB>'s or C<CODE_REF>'s or the transfer
-will fail.
+For classes not included with C<MCE::Shared>, the object must not contain
+any C<GLOB>'s or C<CODE_REF>'s or the transfer will fail.
 
    use MCE::Shared;
    use MCE::Shared::Ordhash;
@@ -812,7 +842,7 @@ elements. Call C<ins_inplace> to update elements.
    my $res2 = $b->slice(":,3:4") + 10;
    $b->ins_inplace($res2, 0, 3);
 
-   # make non-shared, destroy/export
+   # make non-shared object, export-destroy the shared object
    $b = $b->destroy;
 
    print "$b\n";
@@ -849,7 +879,7 @@ The following provides parallel demonstrations using C<MCE::Flow>.
       $b->ins_inplace($result, 0, $row1);
    }, 0, 20 - 1;
 
-   # make non-shared, destroy/export
+   # make non-shared object, export-destroy the shared object
 
    $b = $b->destroy;
 
@@ -991,7 +1021,7 @@ The following example iterates through a shared array in parallel.
 
    # ... do other work ...
 
-   $_->join() for MCE::Hobo->list();
+   MCE::Hobo->waitall();
 
    # Output
 
@@ -1017,8 +1047,7 @@ the amount of traffic to and from the shared-manager process.
    my $pi  = MCE::Shared->scalar( 0.0 );
 
    my $seq = MCE::Shared->sequence(
-      { chunk_size => 200_000, bounds_only => 1 },
-      0, $N - 1
+      { chunk_size => 200_000, bounds_only => 1 }, 0, $N - 1
    );
 
    sub compute_pi {
@@ -1040,7 +1069,7 @@ the amount of traffic to and from the shared-manager process.
 
    # ... do other stuff ...
 
-   $_->join() for MCE::Hobo->list();
+   MCE::Hobo->waitall();
 
    printf "pi = %0.13f\n", $pi->get / $N;
 
@@ -1292,6 +1321,53 @@ obtained from a MacBook Pro (Haswell) running at 2.6 GHz, 1600 MHz RAM.
       OO Interface .. : 0.312 secs.
       threads::shared : 1.061 secs.
 
+Beginning with MCE::Shared 1.809, the C<pipeline> method provides another way.
+Included in C<Array>, C<Cache>, C<Hash>, C<Minidb>, and C<Ordhash>, it combines
+multiple commands for the object to be processed serially. For shared objects,
+the call is made atomically due to single IPC to the shared-manager process.
+
+The C<pipeline> method is fully C<wantarray>-aware and receives a list of
+commands and their arguments. In scalar or list context, it returns data from
+the last command in the pipeline.
+
+   use MCE::Mutex;
+   use MCE::Shared;
+
+   my $mutex = MCE::Mutex->new();
+   my $oh = MCE::Shared->ordhash();
+   my @vals;
+
+   # mutex locking
+
+   $mutex->lock;
+   $oh->set( foo => "a_a" );
+   $oh->set( bar => "b_b" );
+   $oh->set( baz => "c_c" );
+   @vals = $oh->mget( qw/ foo bar baz / );
+   $mutex->unlock;
+
+   # pipeline, same thing done atomically
+
+   @vals = $oh->pipeline(
+      [ "set", foo => "a_a" ],
+      [ "set", bar => "b_b" ],
+      [ "set", baz => "c_c" ],
+      [ "mget", qw/ foo bar baz / ]
+   );
+
+   # ( "a_a", "b_b", "c_c" )
+
+There is also C<pipeline_ex>, same as C<pipeline>, but returns data for every
+command in the pipeline.
+
+   @vals = $oh->pipeline_ex(
+      [ "set", foo => "a_a" ],
+      [ "set", bar => "b_b" ],
+      [ "set", baz => "c_c" ]
+   );
+
+   # ( "a_a", "b_b", "c_c" )
+
 =head1 REQUIREMENTS
 
 MCE::Shared requires Perl 5.10.1 or later. The L<IO::FDPass> module is highly
