@@ -12,11 +12,12 @@ use warnings;
 
 no warnings qw( threads recursion uninitialized numeric );
 
-our $VERSION = '1.808';
+our $VERSION = '1.809';
 
 ## no critic (TestingAndDebugging::ProhibitNoStrict)
 
 use MCE::Shared::Base;
+use parent -norequire, 'MCE::Shared::Base::Common';
 use bytes;
 
 use overload (
@@ -394,21 +395,35 @@ MCE::Shared::Array - Array helper class
 
 =head1 VERSION
 
-This document describes MCE::Shared::Array version 1.808
+This document describes MCE::Shared::Array version 1.809
+
+=head1 DESCRIPTION
+
+An array helper class for use as a standalone or managed by L<MCE::Shared>.
 
 =head1 SYNOPSIS
 
-   # non-shared
+   # non-shared or local construction for use by a single process
+
    use MCE::Shared::Array;
 
    my $ar = MCE::Shared::Array->new( @list );
 
-   # shared
+   # construction for sharing with other threads and processes
+
    use MCE::Shared;
 
    my $ar = MCE::Shared->array( @list );
 
-   # oo interface
+   # array-like dereferencing
+
+   my $val = $ar->[$index];
+   $ar->[$index] = $val;
+
+   @{$ar} = ();
+
+   # OO interface
+
    $val   = $ar->set( $index, $val );
    $val   = $ar->get( $index);
    $val   = $ar->delete( $index );            # del is an alias for delete
@@ -442,7 +457,83 @@ This document describes MCE::Shared::Array version 1.808
    @vals  = $ar->sort( "alpha" );             # $a cmp $b
    @vals  = $ar->sort( "alpha desc" );        # $b cmp $a
 
-   # search capability key/val { =~ !~ eq ne lt le gt ge == != < <= > >= }
+   # included, sugar methods without having to call set/get explicitly
+
+   $len   = $ar->append( $index, $string );   #   $val .= $string
+   $val   = $ar->decr( $index );              # --$val
+   $val   = $ar->decrby( $index, $number );   #   $val -= $number
+   $val   = $ar->getdecr( $index );           #   $val--
+   $val   = $ar->getincr( $index );           #   $val++
+   $val   = $ar->incr( $index );              # ++$val
+   $val   = $ar->incrby( $index, $number );   #   $val += $number
+   $old   = $ar->getset( $index, $new );      #   $o = $v, $v = $n, $o
+
+   # pipeline, provides atomicity for shared objects, MCE::Shared v1.09+
+
+   @vals  = $ar->pipeline(                    # ( "a_a", "b_b", "c_c" )
+      [ "set", 0 => "a_a" ],
+      [ "set", 1 => "b_b" ],
+      [ "set", 2 => "c_c" ],
+      [ "mget", qw/ 0 1 2 / ]
+   );
+
+For normal array behavior, the TIE interface is supported.
+
+   # non-shared or local construction for use by a single process
+
+   use MCE::Shared::Array;
+
+   tie my @ar, "MCE::Shared::Array";
+
+   # construction for sharing with other threads and processes
+
+   use MCE::Shared;
+
+   tie my @ar, "MCE::Shared";
+
+   # usage
+
+   my $val;
+
+   if ( !defined ( $val = $ar[some_index] ) ) {
+      $val = $ar[some_index] = "some_value";
+   }
+
+   $ar[some_index] = 0;
+
+   tied(@ar)->incrby("some_index", 20);
+   tied(@ar)->incrby(some_index => 20);
+
+=head1 SYNTAX for QUERY STRING
+
+Several methods take a query string for an argument. The format of the string
+is described below. In the context of sharing, the query mechanism is beneficial
+for the shared-manager process. It is able to perform the query where the data
+resides versus the client-process greping locally involving lots of IPC.
+
+   o Basic demonstration
+
+     @keys = $ar->keys( "query string given here" );
+     @keys = $ar->keys( "val =~ /pattern/" );
+
+   o Supported operators: =~ !~ eq ne lt le gt ge == != < <= > >=
+   o Multiple expressions delimited by :AND or :OR, mixed case allowed
+
+     "key == 3 :or (val > 5 :and val < 9)"
+     "key =~ /pattern/i :And val =~ /pattern/i"
+     "val eq foo baz :OR key !~ /pattern/i"
+
+     * key matches on indices in the array
+     * likewise, val matches on values
+
+   o Quoting is optional inside the string
+
+     "key =~ /pattern/i :AND val eq 'foo bar'"   # val eq "foo bar"
+     "key =~ /pattern/i :AND val eq foo bar"     # val eq "foo bar"
+
+Examples.
+
+   # search capability key/val: =~ !~ eq ne lt le gt ge == != < <= > >=
    # key/val means to match against actual key/val respectively
 
    @keys  = $ar->keys( "key == 3 :or (val > 5 :and val < 9)" );
@@ -459,58 +550,19 @@ This document describes MCE::Shared::Array version 1.808
    %pairs = $ar->pairs( "val >  $number" );
    %pairs = $ar->pairs( "val >= $number" );
 
-   @vals  = $ar->values( "key eq $string" );
-   @vals  = $ar->values( "key ne $string with space" );
-   @vals  = $ar->values( "key lt $string :or val =~ /$pat1|$pat2/" );
-   @vals  = $ar->values( "val le $string :and val eq 'foo bar'" );
-   @vals  = $ar->values( "val le $string :and val eq foo bar" );
-   @vals  = $ar->values( "val gt $string" );
-   @vals  = $ar->values( "val ge $string" );
-
-   # sugar methods without having to call set/get explicitly
-
-   $len   = $ar->append( $index, $string );   #   $val .= $string
-   $val   = $ar->decr( $index );              # --$val
-   $val   = $ar->decrby( $index, $number );   #   $val -= $number
-   $val   = $ar->getdecr( $index );           #   $val--
-   $val   = $ar->getincr( $index );           #   $val++
-   $val   = $ar->incr( $index );              # ++$val
-   $val   = $ar->incrby( $index, $number );   #   $val += $number
-   $old   = $ar->getset( $index, $new );      #   $o = $v, $v = $n, $o
-
-=head1 DESCRIPTION
-
-An array helper class for use with L<MCE::Shared>.
-
-=head1 SYNTAX for QUERY STRING
-
-Several methods in C<MCE::Shared::Array> take a query string for an argument.
-
-   o Basic demonstration: @keys = $ar->keys( "val =~ /pattern/" );
-   o Supported operators: =~ !~ eq ne lt le gt ge == != < <= > >=
-   o Multiple expressions delimited by :AND or :OR
-   o Quoting optional inside the string
-
-     "key == 3 :or (val > 5 :and val < 9)"
-     "key =~ /pattern/i :and val =~ /pattern/i"
-     "key =~ /pattern/i :and val eq 'foo bar'"   # val eq "foo bar"
-     "key =~ /pattern/i :and val eq foo bar"     # val eq "foo bar"
-     "val eq foo baz :or key !~ /pattern/i"
-
-     * key matches on indices in the array
-     * val matches on values
-
-=over 3
-
-=item * The modifiers C<:AND> and C<:OR> may be mixed case. e.g. C<:And>
-
-=back
+   @vals  = $ar->vals( "key eq $string" );
+   @vals  = $ar->vals( "key ne $string with space" );
+   @vals  = $ar->vals( "key lt $string :or val =~ /$pat1|$pat2/" );
+   @vals  = $ar->vals( "val le $string :and val eq 'foo bar'" );
+   @vals  = $ar->vals( "val le $string :and val eq foo bar" );
+   @vals  = $ar->vals( "val gt $string" );
+   @vals  = $ar->vals( "val ge $string" );
 
 =head1 API DOCUMENTATION
 
 This module may involve TIE when accessing the object via array-like behavior.
 Only shared instances are impacted if doing so. Although likely fast enough for
-many use cases, use the OO interface if better performance is desired.
+many use cases, the OO interface is recommended for best performance.
 
 =over 3
 
@@ -518,13 +570,15 @@ many use cases, use the OO interface if better performance is desired.
 
 Constructs a new object, with an optional list of values.
 
-   # non-shared
+   # non-shared or local construction for use by a single process
+
    use MCE::Shared::Array;
 
    $ar = MCE::Shared::Array->new( @list );
    $ar = MCE::Shared::Array->new( );
 
-   # shared
+   # construction for sharing with other threads and processes
+
    use MCE::Shared;
 
    $ar = MCE::Shared->array( @list );
@@ -717,6 +771,48 @@ described above. In scalar context, returns the size of the resulting list.
    @pairs = $ar->pairs( "val eq sun :OR val eq moon :OR val eq foo" );
    $len   = $ar->pairs( "key =~ /$pattern/" );
 
+=item pipeline ( [ func1, @args ], [ func2, @args ], ... )
+
+Combines multiple commands for the object to be processed serially. For shared
+objects, the call is made atomically due to single IPC to the shared-manager
+process. The C<pipeline> method is fully C<wantarray>-aware and receives a list
+of commands and their arguments. In scalar or list context, it returns data
+from the last command in the pipeline.
+
+   @vals = $ar->pipeline(                     # ( "a_a", "b_b", "c_c" )
+      [ "set", 0 => "a_a" ],
+      [ "set", 1 => "b_b" ],
+      [ "set", 2 => "c_c" ],
+      [ "mget", qw/ 0 1 2 / ]
+   );
+
+   $len = $ar->pipeline(                      # 3, same as $ar->len
+      [ "set", 0 => "i_i" ],
+      [ "set", 1 => "j_j" ],
+      [ "set", 2 => "k_k" ],
+      [ "len" ]
+   );
+
+   $ar->pipeline(
+      [ "set", 0 => "m_m" ],
+      [ "set", 1 => "n_n" ],
+      [ "set", 2 => "o_o" ]
+   );
+
+Current API available since 1.809.
+
+=item pipeline_ex ( [ func1, @args ], [ func2, @args ], ... )
+
+Same as C<pipeline>, but returns data for every command in the pipeline.
+
+   @vals = $ar->pipeline_ex(                  # ( "a_a", "b_b", "c_c" )
+      [ "set", 0 => "a_a" ],
+      [ "set", 1 => "b_b" ],
+      [ "set", 2 => "c_c" ]
+   );
+
+Current API available since 1.809.
+
 =item pop
 
 Removes and returns the last value of the list. If there are no elements in the
@@ -837,7 +933,10 @@ C<vals> is an alias for C<values>.
 =head1 SUGAR METHODS
 
 This module is equipped with sugar methods to not have to call C<set>
-and C<get> explicitly. The API resembles a subset of the Redis primitives
+and C<get> explicitly. In shared context, the benefit is atomicity and
+reduction in inter-process communication.
+
+The API resembles a subset of the Redis primitives
 L<http://redis.io/commands#strings> with key representing the array index.
 
 =over 3
