@@ -12,43 +12,57 @@ use warnings;
 
 no warnings qw( threads recursion uninitialized numeric );
 
-our $VERSION = '1.808';
+our $VERSION = '1.809';
 
 ## no critic (BuiltinFunctions::ProhibitStringyEval)
-## do not remove numeric from no warnings above
+## no critic (Subroutines::ProhibitExplicitReturnUndef)
 
 use Scalar::Util qw( looks_like_number );
 use bytes;
 
-##  o Find feature
 ##
-##  o Used by MCE::Shared::{ Array, Hash, Minidb, and Ordhash }
-##    methods which take a query string for an argument.
+#  Several methods in MCE::Shared::{ Array, Cache, Hash, Minidb, Ordhash } take
+#  a query string for an argument. The format of the string is described below.
+#  The _compile function is where the query string is evaluated and expanded
+#  into Perl code.
+# 
+#  In the context of sharing, the query mechanism is beneficial for the shared-
+#  manager process. The shared-manager performs the query where the data resides
+#  versus sending data in whole to the client process for traversing. Only the
+#  data found is sent.
+# 
+#  o Basic demonstration
+# 
+#    @keys = $oh->keys( "query string given here" );
+#    @keys = $oh->keys( "val =~ /pattern/" );
+# 
+#  o Supported operators: =~ !~ eq ne lt le gt ge == != < <= > >=
+#  o Multiple expressions delimited by :AND or :OR, mixed case allowed
+# 
+#    "key eq 'some key' :or (val > 5 :and val < 9)"
+#    "key eq some key :or (val > 5 :and val < 9)"
+#    "key =~ /pattern/i :And field =~ /pattern/i"
+#    "key =~ /pattern/i :And index =~ /pattern/i"
+#    "index eq 'foo baz' :OR key !~ /pattern/i"    # 9 eq 'foo baz'
+#    "index eq foo baz :OR key !~ /pattern/i"      # 9 eq foo baz
+# 
+#    MCE::Shared::{ Array, Cache, Hash, Ordhash }
+#    * key matches on keys in the hash or index in the array
+#    * likewise, val matches on values
+# 
+#    MCE::Shared::{ Minidb }
+#    * key   matches on primary keys in the hash (H)oH or (H)oA
+#    * field matches on HoH->{key}{field} e.g. address
+#    * index matches on HoA->{key}[index] e.g. 9
+# 
+#  o Quoting is optional inside the string
+#
+#    "key =~ /pattern/i :AND field eq 'foo bar'"   # address eq 'foo bar'
+#    "key =~ /pattern/i :AND field eq foo bar"     # address eq foo bar
+#
+#  o See respective module in section labeled SYNTAX for QUERY STRING
+#    for demonstrations
 ##
-##  o Basic demonstration: @keys = $oh->keys( "val =~ /pattern/" );
-##  o Supported operators: =~ !~ eq ne lt le gt ge == != < <= > >=
-##  o Multiple expressions delimited by :AND or :OR
-##  o Quoting optional inside the string
-##
-##    "key eq 'some key' :or (val > 5 :and val < 9)"
-##    "key eq some key :or (val > 5 :and val < 9)"
-##    "key =~ /pattern/i :and field =~ /pattern/i"
-##    "key =~ /pattern/i :and index =~ /pattern/i"
-##    "key =~ /pattern/i :and field eq 'foo bar'"   # address eq 'foo bar'
-##    "key =~ /pattern/i :and field eq foo bar"     # address eq foo bar
-##    "index eq 'foo baz' :or key !~ /pattern/i"    # 9 eq 'foo baz'
-##    "index eq foo baz :or key !~ /pattern/i"      # 9 eq foo baz
-##
-##    MCE::Shared::{ Array, Hash, Ordhash }
-##    * key matches on keys in the hash or index in the array
-##    * val matches on values
-##
-##    MCE::Shared::Minidb
-##    * key   matches on primary keys in the hash (H)oH or (H)oA
-##    * field matches on HoH->{key}{field} e.g. address
-##    * index matches on HoA->{key}[index] e.g. 9
-##
-##  o The modifiers :AND and :OR may be mixed case. e.g. :And
 
 sub _compile {
    my ( $query ) = @_;
@@ -127,7 +141,7 @@ sub _find_array {
 
 ###############################################################################
 ## ----------------------------------------------------------------------------
-## Find items in HASH. Called by MCE::Shared::{ Hash, Minidb, Ordhash }.
+## Find items in HASH. Called by MCE::Shared::{ Cache, Hash, Minidb, Ordhash }.
 ##
 ###############################################################################
 
@@ -159,7 +173,7 @@ sub _find_hash {
       $q =~ s/(!ref)\(([^\$\)]+)\)/$1(\$data->{\$_}['$2'])/gi;
    }
 
-   # Hash/Ordhash value
+   # Cache/Hash/Ordhash value
    elsif ( $params->{'getvals'} && $q !~ /\(\$_/ ) {
       $grepvals = 1;
       $q =~ s/val[ ]+(==|!=|<|<=|>|>=|eq|ne|lt|le|gt|ge|=~|!~)/\$_ $1/gi;
@@ -243,6 +257,58 @@ sub _die {
    CORE::exit($?);
 }
 
+###############################################################################
+## ----------------------------------------------------------------------------
+## Common API for MCE::Shared::{ Array, Cache, Hash, Minidb, Ordhash }.
+##
+###############################################################################
+
+package MCE::Shared::Base::Common;
+
+use 5.010001;
+use strict;
+use warnings;
+
+no warnings qw( threads recursion uninitialized numeric );
+
+use bytes;
+
+# pipeline ( [ func1, @args ], [ func2, @args ], ... )
+
+sub pipeline {
+   my $self = shift;
+   my $tmp; $tmp = pop if ( defined wantarray );
+
+   while ( @_ ) {
+      my $cmd = shift; next unless ( ref $cmd eq 'ARRAY' );
+      if ( my $code = $self->can(shift @{ $cmd }) ) {
+         $code->($self, @{ $cmd });
+      }
+   }
+
+   if ( defined $tmp ) {
+      my $code;
+      return ( ref $tmp eq 'ARRAY' && ( $code = $self->can(shift @{ $tmp }) ) )
+         ? $code->($self, @{ $tmp })
+         : undef;
+   }
+
+   return;
+}
+
+# pipeline_ex ( [ func1, @args ], [ func2, @args ], ... )
+
+sub pipeline_ex {
+   my $self = shift;
+   my $code;
+
+   map {
+      ( ref $_ eq 'ARRAY' && ( $code = $self->can(shift @{ $_ }) ) )
+         ? $code->($self, @{ $_ })
+         : undef;
+   } @_;
+}
+
 1;
 
 __END__
@@ -259,7 +325,7 @@ MCE::Shared::Base - Base package for helper classes
 
 =head1 VERSION
 
-This document describes MCE::Shared::Base version 1.808
+This document describes MCE::Shared::Base version 1.809
 
 =head1 DESCRIPTION
 
