@@ -12,7 +12,7 @@ use warnings;
 
 no warnings qw( threads recursion uninitialized numeric );
 
-our $VERSION = '1.808';
+our $VERSION = '1.809';
 
 use MCE::Shared::Base;
 use MCE::Util ();
@@ -132,7 +132,12 @@ MCE::Shared::Condvar - Condvar helper class
 
 =head1 VERSION
 
-This document describes MCE::Shared::Condvar version 1.808
+This document describes MCE::Shared::Condvar version 1.809
+
+=head1 DESCRIPTION
+
+This helper class made for L<MCE::Shared> provides a C<Scalar>, C<Mutex>,
+and primitives for conditional locking.
 
 =head1 SYNOPSIS
 
@@ -140,25 +145,25 @@ This document describes MCE::Shared::Condvar version 1.808
 
    my $cv = MCE::Shared->condvar( 0 );
 
-   # oo interface
+   # OO interface
+
    $val = $cv->set( $val );
    $val = $cv->get();
    $len = $cv->len();
 
    # conditional locking primitives
+
    $cv->lock();
    $cv->unlock();
-
-   $cv->broadcast();
    $cv->broadcast(0.05);     # delay before broadcasting
-
-   $cv->signal();
+   $cv->broadcast();
    $cv->signal(0.05);        # delay before signaling
-
+   $cv->signal();
    $cv->timedwait(2.5);
    $cv->wait();
 
-   # sugar methods without having to call set/get explicitly
+   # included, sugar methods without having to call set/get explicitly
+
    $val = $cv->append( $string );     #   $val .= $string
    $val = $cv->decr();                # --$val
    $val = $cv->decrby( $number );     #   $val -= $number
@@ -168,12 +173,9 @@ This document describes MCE::Shared::Condvar version 1.808
    $val = $cv->incrby( $number );     #   $val += $number
    $old = $cv->getset( $new );        #   $o = $v, $v = $n, $o
 
-=head1 DESCRIPTION
+=head1 EXAMPLE
 
-This helper class for L<MCE::Shared> provides a C<Scalar>, C<Mutex>, and
-primitives for conditional locking.
-
-The following demonstrates barrier synchronization.
+The following example demonstrates barrier synchronization.
 
    use MCE;
    use MCE::Shared;
@@ -208,19 +210,12 @@ The following demonstrates barrier synchronization.
       }
    }
 
-   # Time taken from a 2.6 GHz machine running Mac OS X.
-   #
-   # threads::shared:   0.207s  threads
-   #   forks::shared:  36.426s  child processes
-   #     MCE::Shared:   0.353s  child processes
-   #        MCE Sync:   0.062s  child processes
-
    sub user_func {
       my $id = MCE->wid;
       for (1 .. 400) {
          MCE->print("$_: $id\n");
-       # MCE->sync();     # via MCE-Core API
-         barrier_sync();  # via MCE::Shared::Condvar
+         barrier_sync();  # made possible by MCE::Shared::Condvar
+       # MCE->sync();     # same thing via the MCE-Core API
       }
    }
 
@@ -229,6 +224,12 @@ The following demonstrates barrier synchronization.
       user_func   => \&user_func
    )->run;
 
+   # Time taken from a 2.6 GHz machine running Mac OS X.
+   # threads::shared:   0.207s  Perl threads
+   #   forks::shared:  36.426s  child processes
+   #     MCE::Shared:   0.353s  child processes
+   #        MCE Sync:   0.062s  child processes
+
 =head1 API DOCUMENTATION
 
 =over 3
@@ -238,7 +239,6 @@ The following demonstrates barrier synchronization.
 Constructs a new condition variable. Its value defaults to C<0> when C<value>
 is not specified.
 
-   # shared
    use MCE::Shared;
 
    $cv = MCE::Shared->condvar( 100 );
@@ -326,7 +326,10 @@ In either case, the variable is *not* locked upon return.
 =head1 SUGAR METHODS
 
 This module is equipped with sugar methods to not have to call C<set>
-and C<get> explicitly. The API resembles a subset of the Redis primitives
+and C<get> explicitly. In shared context, the benefit is atomicity and
+reduction in inter-process communication.
+
+The API resembles a subset of the Redis primitives
 L<http://redis.io/commands#strings> without the key argument.
 
 =over 3
@@ -387,17 +390,18 @@ The conditional locking aspect is inspired by L<threads::shared>.
 
 =head1 LIMITATION
 
-Perl must have the L<IO::FDPass> module installed for constructing a shared
-C<queue> or C<condvar> while the shared-manager process is running.
-
-For platforms where C<IO::FDPass> is not feasible, construct C<queues> or
-C<condvars> first before other classes. The shared-manager process is delayed
-until sharing other classes or explicitly starting the process.
+Perl must have L<IO::FDPass> for constructing a shared C<condvar>, C<handle>,
+or C<queue> while the shared-manager process is running. For platforms where
+C<IO::FDPass> is not feasible, construct any C<condvar>, C<handle>, and
+C<queue> first before other classes. The shared-manager process is delayed
+until sharing other classes or starting the manager explicitly.
 
    use MCE::Shared;
 
-   my $q1 = MCE::Shared->queue();
-   my $cv = MCE::Shared->condvar();
+   my $cv  = MCE::Shared->condvar();
+   my $que = MCE::Shared->queue();
+
+   mce_open my $fh, ">>", "/path/to/file.log";
 
    MCE::Shared->start();
 
