@@ -12,7 +12,7 @@ use warnings;
 
 no warnings qw( threads recursion uninitialized numeric );
 
-our $VERSION = '1.808';
+our $VERSION = '1.809';
 
 ## no critic (InputOutput::ProhibitTwoArgOpen)
 ## no critic (Subroutines::ProhibitSubroutinePrototypes)
@@ -219,11 +219,16 @@ MCE::Shared::Handle - Handle helper class
 
 =head1 VERSION
 
-This document describes MCE::Shared::Handle version 1.808
+This document describes MCE::Shared::Handle version 1.809
+
+=head1 DESCRIPTION
+
+A handle helper class for use as a standalone or managed by L<MCE::Shared>.
 
 =head1 SYNOPSIS
 
-   # non-shared
+   # non-shared or local construction for use by a single process
+
    use MCE::Shared::Handle;
 
    MCE::Shared::Handle->open( my $fh, "<", "bio.fasta" );
@@ -231,7 +236,8 @@ This document describes MCE::Shared::Handle version 1.808
 
    mce_open my $fh, "<", "bio.fasta" or die "open error: $!";
 
-   # shared
+   # construction for sharing with other threads and processes
+
    use MCE::Shared;
 
    MCE::Shared->open( my $fh, "<", "bio.fasta" );
@@ -239,14 +245,14 @@ This document describes MCE::Shared::Handle version 1.808
 
    mce_open my $fh, "<", "bio.fasta" or die "open error: $!";
 
-   # demo
+   # example, output is serialized, not garbled
+
    use MCE::Hobo;
    use MCE::Shared;
 
    mce_open my $ofh, ">>", \*STDOUT  or die "open error: $!";
    mce_open my $ifh, "<", "file.log" or die "open error: $!";
 
-   # output is serialized (not garbled), but not ordered
    sub parallel {
       $/ = "\n"; # can set the input record separator
       while (my $line = <$ifh>) {
@@ -259,6 +265,7 @@ This document describes MCE::Shared::Handle version 1.808
    $_->join() for MCE::Hobo->list();
 
    # handle functions
+
    my $bool = eof($ifh);
    my $off  = tell($ifh);
    my $fd   = fileno($ifh);
@@ -266,7 +273,6 @@ This document describes MCE::Shared::Handle version 1.808
    my $line = readline($ifh);
 
    binmode $ifh;
-
    seek $ifh, 10, 0;
    read $ifh, my($buf), 80;
 
@@ -279,10 +285,6 @@ This document describes MCE::Shared::Handle version 1.808
    close $ifh;
    close $ofh;
 
-=head1 DESCRIPTION
-
-Helper class for L<MCE::Shared>.
-
 =head1 API DOCUMENTATION
 
 =over 3
@@ -298,23 +300,29 @@ whose filename is given by C<expr>, and associates it with C<filehandle>.
 When omitting error checking at the application level, MCE::Shared emits
 a message and stop if open fails.
 
-   # non-shared
+   # non-shared or local construction for use by a single process
+
    use MCE::Shared::Handle;
 
    MCE::Shared::Handle->open( my $fh, "<", "file.log" ) or die "$!";
    MCE::Shared::Handle::open  my $fh, "<", "file.log"   or die "$!";
 
-   # shared
+   mce_open my $fh, "<", "file.log" or die "$!"; # ditto
+
+   # construction for sharing with other threads and processes
+
    use MCE::Shared;
 
    MCE::Shared->open( my $fh, "<", "file.log" ) or die "$!";
    MCE::Shared::open  my $fh, "<", "file.log"   or die "$!";
 
+   mce_open my $fh, "<", "file.log" or die "$!"; # ditto
+
 Simple examples to open a file for reading:
 
-   # mce_open, exported
-   # creates a shared handle when MCE::Shared is present
-   # creates a non-shared handle, otherwise
+   # mce_open is exported by MCE::Shared or MCE::Shared::Handle.
+   # It creates a shared file handle with MCE::Shared present
+   # or a non-shared handle otherwise.
 
    mce_open my $fh, "< input.txt"     or die "open error: $!";
    mce_open my $fh, "<", "input.txt"  or die "open error: $!";
@@ -332,11 +340,9 @@ and for writing:
 
 Starting with C<MCE::Shared> v1.007, chunk IO is possible for both non-shared
 and shared handles. Chunk IO is enabled by the trailing 'k' or 'm' for read
-size.
-
-Also, chunk IO supports the special "\n>"-like record separator. That anchors
-">" at the start of the line. Workers receive record(s) beginning with ">" and
-ending with "\n".
+size. Also, chunk IO supports the special "\n>"-like record separator.
+That anchors ">" at the start of the line. Workers receive record(s) beginning
+with ">" and ending with "\n".
 
    # non-shared handle ---------------------------------------------
 
@@ -350,7 +356,7 @@ ending with "\n".
 
    mce_open my $fh, '<', 'bio.fasta' or die "open error: $!";
 
-   # 'k' or 'm' indicates kilobytes or megabytes respectively.
+   # 'k' or 'm' indicates kibiBytes (KiB) or mebiBytes (MiB) respectively.
    # Read continues reading until reaching the record separator or EOF.
    # Optionally, one may specify the record separator.
 
@@ -443,13 +449,25 @@ The benefit of chunk IO is from lesser IPC for the shared-manager process
 
 =head1 LIMITATION
 
-When passing a C<reference>, be sure to construct its C<file handle> associated
-with C<reference> prior to starting the shared-manager process. Constructing a
-shared object C<{ Array, Handle, Hash, Minidb, Ordhash, Scalar, Sequence }>
-starts the manager process automatically.
+Perl must have L<IO::FDPass> for constructing a shared C<condvar>, C<handle>,
+or C<queue> while the shared-manager process is running. For platforms where
+C<IO::FDPass> is not feasible, construct any C<condvar>, C<handle>, and
+C<queue> first before other classes. The shared-manager process is delayed
+until sharing other classes or starting the manager explicitly.
 
-Perl must have the L<IO::FDPass> module installed for C<MCE::Shared> to pass a
-C<file_descriptor> higher than 2 to the shared-manager process.
+   use MCE::Shared;
+
+   my $cv  = MCE::Shared->condvar();
+   my $que = MCE::Shared->queue();
+
+   mce_open my $fh, ">>", "/path/to/file.log";
+
+   MCE::Shared->start();
+
+When passing a C<reference>, be sure to construct its C<file handle> associated
+with C<reference> prior to the shared-manager process being spawned.
+
+   mce_open my $fh, ">>", \*non_shared_fh;
 
 =head1 CREDITS
 
