@@ -12,11 +12,12 @@ use warnings;
 
 no warnings qw( threads recursion uninitialized numeric );
 
-our $VERSION = '1.808';
+our $VERSION = '1.809';
 
 ## no critic (TestingAndDebugging::ProhibitNoStrict)
 
 use MCE::Shared::Base;
+use parent -norequire, 'MCE::Shared::Base::Common';
 use bytes;
 
 use overload (
@@ -310,21 +311,39 @@ MCE::Shared::Hash - Hash helper class
 
 =head1 VERSION
 
-This document describes MCE::Shared::Hash version 1.808
+This document describes MCE::Shared::Hash version 1.809
+
+=head1 DESCRIPTION
+
+A hash helper class for use as a standalone or managed by L<MCE::Shared>.
 
 =head1 SYNOPSIS
 
-   # non-shared
+   # non-shared or local construction for use by a single process
+
    use MCE::Shared::Hash;
 
    my $ha = MCE::Shared::Hash->new( @pairs );
 
-   # shared
+   # construction for sharing with other threads and processes
+
    use MCE::Shared;
 
    my $ha = MCE::Shared->hash( @pairs );
 
-   # oo interface
+   # hash-like dereferencing
+
+   my $val = $ha->{$key};
+   $ha->{$key} = $val;
+
+   %{$ha} = ();
+
+   # OO interface
+
+   if ( !defined ( $val = $ha->get("some_key") ) ) {
+      $val = $ha->set( some_key => "some_value" );
+   }
+
    $val   = $ha->set( $key, $val );
    $val   = $ha->get( $key );
    $val   = $ha->delete( $key );              # del is an alias for delete
@@ -346,7 +365,84 @@ This document describes MCE::Shared::Hash version 1.808
    $bool  = $ha->mexists( @keys );            # true if all keys exists
    $len   = $ha->mset( $key/$val pairs );     # merge is an alias for mset
 
-   # search capability key/val { =~ !~ eq ne lt le gt ge == != < <= > >= }
+   # included, sugar methods without having to call set/get explicitly
+
+   $len   = $ha->append( $key, $string );     #   $val .= $string
+   $val   = $ha->decr( $key );                # --$val
+   $val   = $ha->decrby( $key, $number );     #   $val -= $number
+   $val   = $ha->getdecr( $key );             #   $val--
+   $val   = $ha->getincr( $key );             #   $val++
+   $val   = $ha->incr( $key );                # ++$val
+   $val   = $ha->incrby( $key, $number );     #   $val += $number
+   $old   = $ha->getset( $key, $new );        #   $o = $v, $v = $n, $o
+
+   # pipeline, provides atomicity for shared objects, MCE::Shared v1.09+
+
+   @vals  = $ha->pipeline(                    # ( "a_a", "b_b", "c_c" )
+      [ "set", foo => "a_a" ],
+      [ "set", bar => "b_b" ],
+      [ "set", baz => "c_c" ],
+      [ "mget", qw/ foo bar baz / ]
+   );
+
+For normal hash behavior, the TIE interface is supported.
+
+   # non-shared or local construction for use by a single process
+
+   use MCE::Shared::Hash;
+
+   tie my %ha, "MCE::Shared::Hash";
+
+   # construction for sharing with other threads and processes
+
+   use MCE::Shared;
+
+   tie my %ha, "MCE::Shared";
+
+   # usage
+
+   my $val;
+
+   if ( !defined ( $val = $ha{some_key} ) ) {
+      $val = $ha{some_key} = "some_value";
+   }
+
+   $ha{some_key} = 0;
+
+   tied(%ha)->incrby("some_key", 20);
+   tied(%ha)->incrby(some_key => 20);
+
+=head1 SYNTAX for QUERY STRING
+
+Several methods take a query string for an argument. The format of the string
+is described below. In the context of sharing, the query mechanism is beneficial
+for the shared-manager process. It is able to perform the query where the data
+resides versus the client-process greping locally involving lots of IPC.
+
+   o Basic demonstration
+
+     @keys = $ha->keys( "query string given here" );
+     @keys = $ha->keys( "val =~ /pattern/" );
+
+   o Supported operators: =~ !~ eq ne lt le gt ge == != < <= > >=
+   o Multiple expressions delimited by :AND or :OR, mixed case allowed
+
+     "key eq 'some key' :or (val > 5 :and val < 9)"
+     "key eq some key :or (val > 5 :and val < 9)"
+     "key =~ /pattern/i :And val =~ /pattern/i"
+     "val eq foo baz :OR key !~ /pattern/i"
+
+     * key matches on keys in the hash
+     * likewise, val matches on values
+
+   o Quoting is optional inside the string
+
+     "key =~ /pattern/i :AND val eq 'foo bar'"   # val eq "foo bar"
+     "key =~ /pattern/i :AND val eq foo bar"     # val eq "foo bar"
+
+Examples.
+
+   # search capability key/val: =~ !~ eq ne lt le gt ge == != < <= > >=
    # key/val means to match against actual key/val respectively
 
    @keys  = $ha->keys( "key eq 'some key' :or (val > 5 :and val < 9)" );
@@ -364,59 +460,19 @@ This document describes MCE::Shared::Hash version 1.808
    %pairs = $ha->pairs( "val >  $number" );
    %pairs = $ha->pairs( "val >= $number" );
 
-   @vals  = $ha->values( "key eq $string" );
-   @vals  = $ha->values( "key ne $string with space" );
-   @vals  = $ha->values( "key lt $string :or val =~ /$pat1|$pat2/" );
-   @vals  = $ha->values( "val le $string :and val eq 'foo bar'" );
-   @vals  = $ha->values( "val le $string :and val eq foo bar" );
-   @vals  = $ha->values( "val gt $string" );
-   @vals  = $ha->values( "val ge $string" );
-
-   # sugar methods without having to call set/get explicitly
-
-   $len   = $ha->append( $key, $string );     #   $val .= $string
-   $val   = $ha->decr( $key );                # --$val
-   $val   = $ha->decrby( $key, $number );     #   $val -= $number
-   $val   = $ha->getdecr( $key );             #   $val--
-   $val   = $ha->getincr( $key );             #   $val++
-   $val   = $ha->incr( $key );                # ++$val
-   $val   = $ha->incrby( $key, $number );     #   $val += $number
-   $old   = $ha->getset( $key, $new );        #   $o = $v, $v = $n, $o
-
-=head1 DESCRIPTION
-
-A hash helper class for use with L<MCE::Shared>.
-
-=head1 SYNTAX for QUERY STRING
-
-Several methods in C<MCE::Shared::Hash> take a query string for an argument.
-
-   o Basic demonstration: @keys = $ha->keys( "val =~ /pattern/" );
-   o Supported operators: =~ !~ eq ne lt le gt ge == != < <= > >=
-   o Multiple expressions delimited by :AND or :OR
-   o Quoting optional inside the string
-
-     "key eq 'some key' :or (val > 5 :and val < 9)"
-     "key eq some key :or (val > 5 :and val < 9)"
-     "key =~ /pattern/i :and val =~ /pattern/i"
-     "key =~ /pattern/i :and val eq 'foo bar'"   # val eq "foo bar"
-     "key =~ /pattern/i :and val eq foo bar"     # val eq "foo bar"
-     "val eq foo baz :or key !~ /pattern/i"
-
-     * key matches on keys in the hash
-     * val matches on values
-
-=over 3
-
-=item * The modifiers C<:AND> and C<:OR> may be mixed case. e.g. C<:And>
-
-=back
+   @vals  = $ha->vals( "key eq $string" );
+   @vals  = $ha->vals( "key ne $string with space" );
+   @vals  = $ha->vals( "key lt $string :or val =~ /$pat1|$pat2/" );
+   @vals  = $ha->vals( "val le $string :and val eq 'foo bar'" );
+   @vals  = $ha->vals( "val le $string :and val eq foo bar" );
+   @vals  = $ha->vals( "val gt $string" );
+   @vals  = $ha->vals( "val ge $string" );
 
 =head1 API DOCUMENTATION
 
 This module may involve TIE when accessing the object via hash-like behavior.
 Only shared instances are impacted if doing so. Although likely fast enough for
-many use cases, use the OO interface if better performance is desired.
+many use cases, the OO interface is recommended for best performance.
 
 =over 3
 
@@ -424,13 +480,15 @@ many use cases, use the OO interface if better performance is desired.
 
 Constructs a new object, with an optional list of key-value pairs.
 
-   # non-shared
+   # non-shared or local construction for use by a single process
+
    use MCE::Shared::Hash;
 
    $ha = MCE::Shared::Hash->new( @pairs );
    $ha = MCE::Shared::Hash->new( );
 
-   # shared
+   # construction for sharing with other threads and processes
+
    use MCE::Shared;
 
    $ha = MCE::Shared->hash( @pairs );
@@ -615,6 +673,48 @@ described above. In scalar context, returns the size of the resulting list.
    @pairs = $ha->pairs( "val eq sun :OR val eq moon :OR val eq foo" );
    $len   = $ha->pairs( "key =~ /$pattern/" );
 
+=item pipeline ( [ func1, @args ], [ func2, @args ], ... )
+
+Combines multiple commands for the object to be processed serially. For shared
+objects, the call is made atomically due to single IPC to the shared-manager
+process. The C<pipeline> method is fully C<wantarray>-aware and receives a list
+of commands and their arguments. In scalar or list context, it returns data
+from the last command in the pipeline.
+
+   @vals = $ha->pipeline(                     # ( "a_a", "b_b", "c_c" )
+      [ "set", foo => "a_a" ],
+      [ "set", bar => "b_b" ],
+      [ "set", baz => "c_c" ],
+      [ "mget", qw/ foo bar baz / ]
+   );
+
+   $len = $ha->pipeline(                      # 3, same as $ha->len
+      [ "set", foo => "i_i" ],
+      [ "set", bar => "j_j" ],
+      [ "set", baz => "k_k" ],
+      [ "len" ]
+   );
+
+   $ha->pipeline(
+      [ "set", foo => "m_m" ],
+      [ "set", bar => "n_n" ],
+      [ "set", baz => "o_o" ]
+   );
+
+Current API available since 1.809.
+
+=item pipeline_ex ( [ func1, @args ], [ func2, @args ], ... )
+
+Same as C<pipeline>, but returns data for every command in the pipeline.
+
+   @vals = $ha->pipeline_ex(                  # ( "a_a", "b_b", "c_c" )
+      [ "set", foo => "a_a" ],
+      [ "set", bar => "b_b" ],
+      [ "set", baz => "c_c" ]
+   );
+
+Current API available since 1.809.
+
 =item set ( key, value )
 
 Sets the value of the given hash key and returns its new value.
@@ -656,7 +756,10 @@ C<vals> is an alias for C<values>.
 =head1 SUGAR METHODS
 
 This module is equipped with sugar methods to not have to call C<set>
-and C<get> explicitly. The API resembles a subset of the Redis primitives
+and C<get> explicitly. In shared context, the benefit is atomicity and
+reduction in inter-process communication.
+
+The API resembles a subset of the Redis primitives
 L<http://redis.io/commands#strings> with key representing the hash key.
 
 =over 3
