@@ -12,13 +12,15 @@ use warnings;
 
 no warnings qw( threads recursion uninitialized numeric );
 
-our $VERSION = '1.808';
+our $VERSION = '1.809';
 
 use MCE::Shared::Base;
+use parent -norequire, 'MCE::Shared::Base::Common';
+use bytes;
+
 use MCE::Shared::Ordhash;
 use MCE::Shared::Array;
 use MCE::Shared::Hash;
-use bytes;
 
 use overload (
    q("")    => \&MCE::Shared::Base::_stringify,
@@ -30,7 +32,7 @@ sub new {
    # Parallel Hashes: [ HoH, HoA ]
    bless [
       MCE::Shared::Ordhash->new(),  # Hash of Hashes (HoH)
-      MCE::Shared::Ordhash->new(),  # Hash of Arrays (HoA)
+      MCE::Shared::Ordhash->new(),  # Hash of Lists (HoA)
    ], shift;
 }
 
@@ -98,33 +100,33 @@ sub _new_list {
    MCE::Shared::Array->new();
 }
 
-# o The select_aref and select_href methods take a select string supporting
-#   field names or list indices and optionally sort modifiers. The syntax for
-#   the query string, between :WHERE and :ORDER BY, is the same as described
-#   in the documentation under the section SYNTAX for QUERY STRING.
+# _qparse ( "select string" )
 #
-# o HoH
-#    "f1 f2 f3 :WHERE f4 > 20 :AND key =~ /foo/ :ORDER BY f5 DESC ALPHA"
-#    "f5 f1 f2 :WHERE fN > 40 :AND key =~ /bar/ :ORDER BY key ALPHA"
-#    "f5 f1 f2 :WHERE fN > 40 :AND key =~ /bar/"
-#    "f5 f1 f2"
+# The select_aref and select_href methods take a select string supporting
+# field names or list indices and optionally sort modifiers. The syntax for
+# the query string, between :WHERE and :ORDER BY, is the same as described
+# in the documentation under the section labeled SYNTAX for QUERY STRING.
+#
+# The modifiers :WHERE, :AND, :OR, ORDER BY, ASC, DESC, ALPHA may be written
+# using mixed case. e.g. :Where
+#
+# o Hash of Hashes (HoH)
+#   "f1 f2 f3 :WHERE f4 > 20 :AND key =~ /foo/ :ORDER BY f5 DESC ALPHA"
+#   "f5 f1 f2 :WHERE fN > 40 :AND key =~ /bar/ :ORDER BY key ALPHA"
+#   "f5 f1 f2 :WHERE fN > 40 :AND key =~ /bar/"
+#   "f5 f1 f2"
 #
 #    * key matches on keys stored in the primary level hash (H)oH
 #
-# o HoA
-#    "17 15 11 :WHERE 12 > 20 :AND key =~ /foo/ :ORDER BY 10 DESC ALPHA"
-#    "17 15 11 :WHERE 12 > 40 :AND key =~ /bar/ :ORDER BY key ALPHA"
-#    "17 15 11 :WHERE 12 > 40 :AND key =~ /bar/"
-#    "17 15 11"
+# o Hash of Lists (HoA)
+#   "17 15 11 :where 12 > 20 :and key =~ /foo/ :order by 10 desc alpha"
+#   "17 15 11 :where 12 > 40 :and key =~ /bar/ :order by key alpha"
+#   "17 15 11 :where 12 > 40 :and key =~ /bar/"
+#   "17 15 11"
 #
-#    * key matches on keys stored in the primary level hash (H)oA
-#    * above, list indices are given as 17, 15, 11, 12, and 10
-#    * the shorter form is allowed e.g. "4 > 20 :AND key =~ /baz/"
-#
-# o The modifiers :WHERE, :AND, :OR, ORDER BY, ASC, DESC, and ALPHA may be
-#   mixed case. e.g. :Where
-
-# _qparse ( "select string" )
+#   * key matches on keys stored in the primary level hash (H)oA
+#   * above, list indices are given as 17, 15, 11, 12, and 10
+#   * the shorter form is allowed e.g. "4 > 20 :AND key =~ /baz/"
 
 sub _qparse {
    my ( $q ) = @_;
@@ -151,8 +153,9 @@ sub _qparse {
    return ( $f, $w, $o );
 }
 
-# _hselect_aref ( "select string" ) see _qparse
-# this returns array containing [ key, aref ] pairs
+# _hselect_aref ( "select string" ), see _qparse for description
+#
+#  returns an array containing [ key, aref ] pairs
 
 sub _hselect_aref {
    my ( $self, $query ) = @_;
@@ -183,8 +186,9 @@ sub _hselect_aref {
    }
 }
 
-# _hselect_href ( "select string" ) see _qparse
-# this returns array containing [ key, href ] pairs
+# _hselect_href ( "select string" ), see _qparse for description
+#
+#  returns an array containing [ key, href ] pairs
 
 sub _hselect_href {
    my ( $self, $query ) = @_;
@@ -222,8 +226,9 @@ sub _hselect_href {
    }
 }
 
-# _lselect_aref ( "select string" ) see _qparse
-# this returns array containing [ key, aref ] pairs
+# _lselect_aref ( "select string" ), see _qparse for description
+#
+#  returns an array containing [ key, aref ] pairs
 
 sub _lselect_aref {
    my ( $self, $query ) = @_;
@@ -261,8 +266,9 @@ sub _lselect_aref {
    }
 }
 
-# _lselect_href ( "select string" ) see _qparse
-# this returns array containing [ key, href ] pairs
+# _lselect_href ( "select string" ), see _qparse for description
+#
+#  returns an array containing [ key, href ] pairs
 
 sub _lselect_href {
    my ( $self, $query ) = @_;
@@ -765,7 +771,7 @@ sub hlen {
 
 ###############################################################################
 ## ----------------------------------------------------------------------------
-## Hash of Arrays (HoA).
+## Hash of Lists (HoA).
 ##
 ###############################################################################
 
@@ -1108,82 +1114,156 @@ MCE::Shared::Minidb - A pure-Perl in-memory data store
 
 =head1 VERSION
 
-This document describes MCE::Shared::Minidb version 1.808
+This document describes MCE::Shared::Minidb version 1.809
+
+=head1 DESCRIPTION
+
+A tiny in-memory NoSQL-like database for use as a standalone or managed by
+L<MCE::Shared>.
+
+This module was created mainly for having an efficient manner in which to
+manipulate hashes-of-hashes (HoH) and hashes-of-lists (HoA) structures with
+MCE::Shared. An application may choose to use both structures or one or the
+other. Internally, both structures reside in memory simultaneously.
+
+   sub new {
+      # Dual top-level hashes: [ HoH, HoA ]
+      bless [
+         MCE::Shared::Ordhash->new(),  # Stores Hash-of-Hashes (HoH)
+         MCE::Shared::Ordhash->new(),  # Stores Hash-of-Lists  (HoA)
+      ], shift;
+   }
+
+   # each Ho(H) key => MCE::Shared::Hash->new()
+   # each Ho(A) key => MCE::Shared::Array->new()
+
+Several methods described below may resemble the C<Redis> API. It is not the
+intent for this module to become 100% compatible.
 
 =head1 SYNOPSIS
 
-   # non-shared
+   # non-shared or local construction for use by a single process
+
    use MCE::Shared::Minidb;
 
    my $db = MCE::Shared::Minidb->new();
 
-   # shared
+   # construction for sharing with other threads and processes
+
    use MCE::Shared;
 
    my $db = MCE::Shared->minidb();
 
-   # HoH
+   # Hash of Hashes (HoH)
+   # Update the hash stored at key1/key2
+
    $db->hset( "key1", "f1", "foo" );
    $db->hset( "key2", "f1", "bar", "f2", "baz" );
 
    $val = $db->hget( "key2", "f2" );  # "baz"
 
-   # HoA
+   # Hash of Lists (HoA)
+   # Update the list stored at key1/key2
+
    $db->lset( "key1", 0, "foo" );
    $db->lset( "key2", 0, "bar", 1, "baz" );
 
    $val = $db->lget( "key2", 1 );     # "baz"
 
-=head1 DESCRIPTION
+   # pipeline, provides atomicity for shared objects, MCE::Shared v1.09+
 
-A tiny in-memory NoSQL-like database for use with L<MCE::Shared>. Although
-several methods resemble the C<Redis> API, it is not the intent for this
-module to become 100% compatible with it.
-
-This module was created mainly for having an efficient manner in which to
-manipulate hashes-of-hashes (HoH) and hashes-of-arrays (HoA) structures with
-MCE::Shared. Both are supported simultaneously due to being unique objects
-inside the C<$db> object.
-
-   sub new {
-      # Parallel Hashes: [ HoH, HoA ]
-      bless [
-         MCE::Shared::Ordhash->new(),  # Hash of Hashes (HoH)
-         MCE::Shared::Ordhash->new(),  # Hash of Arrays (HoA)
-      ], shift;
-   }
-
-   # Ho(H) key => MCE::Shared::Hash->new();
-   # Ho(A) key => MCE::Shared::Array->new()
+   $db->pipeline(
+      [ "lset", "key1", 0, "foo" ],
+      [ "hset", "key1", "field1", "foo" ],
+      [ "lset", "key2", 0, "bar", 1, "baz" ],
+      [ "hset", "key2", "field1", "bar", "field2", "baz" ]
+   );
 
 =head1 SYNTAX for QUERY STRING
 
-Several methods in C<MCE::Shared::Minidb> take a query string.
+Several methods take a query string for an argument. The format of the string
+is described below. In the context of sharing, the query mechanism is beneficial
+for the shared-manager process. It is able to perform the query where the data
+resides versus the client-process greping locally involving lots of IPC.
+
+   o Basic demonstration
+
+     # query the hash stored at "some key"
+     @keys = $db->hkeys( "some key", "query string given here" );
+     @keys = $db->hkeys( "some key", "val =~ /pattern/" );
+
+     # query the top-level hash (H)oH
+     @keys = $db->hkeys( "query string given here" );
+     @keys = $db->hkeys( "key =~ /pattern/" );
 
-   o Basic demonstration: @keys = $db->hkeys( "key", "val =~ /pattern/" );
    o Supported operators: =~ !~ eq ne lt le gt ge == != < <= > >=
-   o Multiple expressions delimited by :AND or :OR
-   o Quoting optional inside the string
+   o Multiple expressions delimited by :AND or :OR, mixed case allowed
   
      "key eq 'some key' :or (field > 5 :and field < 9)"
      "key eq some key :or (field > 5 :and field < 9)"
-     "key =~ /pattern/i :and field =~ /pattern/i"
-     "key =~ /pattern/i :and index =~ /pattern/i"
-     "key =~ /pattern/i :and field eq 'foo bar'"   # address eq "foo bar"
-     "key =~ /pattern/i :and field eq foo bar"     # address eq "foo bar"
-     "index eq foo baz :or key !~ /pattern/i"      # 9 eq "foo baz"
+     "key =~ /pattern/i :And field =~ /pattern/i"   # HoH
+     "key =~ /pattern/i :And index =~ /pattern/i"   # HoA
+     "index eq foo baz :OR key !~ /pattern/i"       # e.g. 9 eq "foo baz"
 
      * key   matches on primary keys in the hash (H)oH or (H)oA
      * field matches on HoH->{key}{field} e.g. address
      * index matches on HoA->{key}[index] e.g. 9
 
-=over 3
+   o Quoting is optional inside the string
+   o Primary keys (H)oH may have spaces, but not so for field_names
 
-=item * Primary keys (H)oH may have spaces, but not secondary field names.
+     "key =~ /pattern/i :AND field eq 'foo bar'"    # address eq "foo bar"
+     "key =~ /pattern/i :AND field eq foo bar"      # address eq "foo bar"
 
-=item * The modifiers C<:AND> and C<:OR> may be mixed case. e.g. C<:And>
+     "key =~ 'some key' :AND 'some_field' eq 'foo bar'"  # ok: some_field
+     "key =~ some key :AND some_field eq foo bar"
 
-=back
+     "key =~ 'some key' :AND 'some field' eq 'foo bar'"  # fail: some field
+     "key =~ some key :AND some field eq foo bar"
+
+Examples.
+
+   # search capability key/val: =~ !~ eq ne lt le gt ge == != < <= > >=
+   # key/val means to match against actual key/val respectively
+
+   # a query made to a hash stored at "some key"
+   # note that "fieldNames" must not have spaces
+
+     @keys  = $db->hkeys(
+        "some key", "key eq some_field :or (val > 5 :and val < 9)"
+     );
+
+   # queries made to a list stored at "some key"
+
+     @pairs = $db->lpairs( "some key", "key >= 50 :AND val =~ /sun|moon/" );
+     @pairs = $db->lpairs( "some key", "val eq sun :OR val eq moon" );
+
+   # the key modifier is the only thing possible for top-level queries
+   # reason: value equals a hash or list reference containing 2nd-level data
+
+     @keys  = $db->hkeys( "key eq 'some key'" );
+     @keys  = $db->hkeys( "key eq some key" );
+
+     @keys  = $db->hkeys( "key =~ /$pattern/i" );
+     @keys  = $db->hkeys( "key !~ /$pattern/i" );
+
+     %pairs = $db->hpairs( "key == $number" );
+     %pairs = $db->hpairs( "key != $number" );
+     %pairs = $db->hpairs( "key <  $number :or key > $number" );
+     %pairs = $db->hpairs( "key <= $number" );
+     %pairs = $db->hpairs( "key >  $number" );
+     %pairs = $db->hpairs( "key >= $number" );
+
+     @vals  = $db->hvals( "key eq $string" );
+     @vals  = $db->hvals( "key ne $string with space" );
+     @vals  = $db->hvals( "key lt $string :or key =~ /$pat1|$pat2/" );
+     @vals  = $db->hvals( "key le $string :or key eq 'foo bar'" );
+     @vals  = $db->hvals( "key le $string :or key eq foo bar" );
+     @vals  = $db->hvals( "key gt $string" );
+     @vals  = $db->hvals( "key ge $string" );
+
+   # see select_aref and select_href below for db-like queries against
+   # the underlying Ho(A) and Ho(H) structures respectively
 
 =head1 API DOCUMENTATION - DB
 
@@ -1193,12 +1273,14 @@ Several methods in C<MCE::Shared::Minidb> take a query string.
 
 Constructs an empty in-memory C<HoH> and C<HoA> key-store database structure.
 
-   # non-shared
+   # non-shared or local construction for use by a single process
+
    use MCE::Shared::Minidb;
 
    $db = MCE::Shared::Minidb->new();
 
-   # shared
+   # construction for sharing with other threads and processes
+
    use MCE::Shared;
 
    $db = MCE::Shared->minidb();
@@ -1209,6 +1291,60 @@ Dumps the in-memory content to a file.
 
    $db->dump( "content.dat" );
 
+=item pipeline ( [ func1, @args ], [ func2, @args ], ... )
+
+Combines multiple commands for the object to be processed serially. For shared
+objects, the call is made atomically due to single IPC to the shared-manager
+process. The C<pipeline> method is fully C<wantarray>-aware and receives a list
+of commands and their arguments. In scalar or list context, it returns data
+from the last command in the pipeline.
+
+   @vals = $db->pipeline(                     # ( "bar", "baz" )
+      [ "hset", "some_key", "f1", "bar", "f2", "baz" ],
+      [ "hget", "some_key", "f1", "f2" ]
+   );
+
+   $len = $db->pipeline(                      # 2, same as $db->hlen("key2)
+      [ "hset", "some_key", "f1", "bar", "f2", "baz" ],
+      [ "hlen", "some_key" ]
+   );
+
+   $db->pipeline(
+      [ "lset", "key1", 0, "foo" ],
+      [ "hset", "key1", "field1", "foo" ],
+      [ "lset", "key2", 0, "bar", 1, "baz" ],
+      [ "hset", "key2", "field1", "bar", "field2", "baz" ]
+   );
+
+Current API available since 1.809.
+
+=item pipeline_ex ( [ func1, @args ], [ func2, @args ], ... )
+
+Same as C<pipeline>, but returns data for every command in the pipeline.
+
+   @vals = $db->pipeline_ex(                  # ( "bar", "baz" )
+      [ "hset", "key3", "field1", "bar" ],
+      [ "hset", "key3", "field2", "baz" ]
+   );
+
+   $chunk_size = 3;
+
+   $db->hset("some_key", chunk_id => 0);
+   $db->lassign("some_key", @ARGV);
+
+   while (1) {
+      ($chunk_id, @next) = $db->pipeline_ex(
+         [ "hincr",   "some_key", "chunk_id"     ],
+         [ "lsplice", "some_key", 0, $chunk_size ]
+      );
+
+      last unless @next;
+
+      ...
+   }
+
+Current API available since 1.809.
+
 =item restore ( "file.dat" )
 
 Restores the in-memory content from a file.
@@ -1543,6 +1679,10 @@ wanting any result.
 
 =head1 SUGAR METHODS - HASHES ( HoH )
 
+This module is equipped with sugar methods to not have to call C<set>
+and C<get> explicitly. In shared context, the benefit is atomicity and
+reduction in inter-process communication.
+
 =over 3
 
 =item happend ( key, field, string )
@@ -1905,6 +2045,10 @@ returns the new length.
 
 =head1 SUGAR METHODS - LISTS ( HoA )
 
+This module is equipped with sugar methods to not have to call C<set>
+and C<get> explicitly. In shared context, the benefit is atomicity and
+reduction in inter-process communication.
+
 =over 3
 
 =item lappend ( key, index, string )
