@@ -24,12 +24,13 @@ use warnings;
 
 no warnings qw( threads recursion uninitialized numeric );
 
-our $VERSION = '1.808';
+our $VERSION = '1.809';
 
 ## no critic (Subroutines::ProhibitExplicitReturnUndef)
 ## no critic (TestingAndDebugging::ProhibitNoStrict)
 
 use MCE::Shared::Base;
+use parent -norequire, 'MCE::Shared::Base::Common';
 use bytes;
 
 # for marking a key to be garbage collected later
@@ -822,21 +823,52 @@ MCE::Shared::Ordhash - An ordered hash class featuring tombstone deletion
 
 =head1 VERSION
 
-This document describes MCE::Shared::Ordhash version 1.808
+This document describes MCE::Shared::Ordhash version 1.809
+
+=head1 DESCRIPTION
+
+An ordered-hash helper class for use as a standalone or managed by
+L<MCE::Shared>.
+
+This module implements an ordered hash featuring tombstone deletion,
+inspired by L<Hash::Ordered>. An ordered hash is very much like a normal
+hash but with key insertion order preserved.
+
+It provides C<splice>, sorting, plus extra capabilities for use with
+L<MCE::Shared::Minidb>. Tombstone deletion is further optimized to not
+impact C<store>, C<push>, C<unshift>, and C<merge>. Tombstones are
+purged in-place for lesser memory consumption. In addition, C<pop> and
+C<shift> run optimally when an index is present. The optimization also
+applies to forward and reverse deletes. The end result is achieving a
+new level of performance, for a pure-Perl ordered hash implementation.
 
 =head1 SYNOPSIS
 
-   # non-shared
+   # non-shared or local construction for use by a single process
+
    use MCE::Shared::Ordhash;
 
    my $oh = MCE::Shared::Ordhash->new( @pairs );
 
-   # shared
+   # construction for sharing with other threads and processes
+
    use MCE::Shared;
 
    my $oh = MCE::Shared->ordhash( @pairs );
 
-   # oo interface
+   # hash-like dereferencing
+
+   my $val = $oh->{$key};
+   $oh->{$key} = $val;
+
+   %{$oh} = ();
+
+   # OO interface
+
+   if ( !defined ( $val = $oh->get("some_key") ) ) {
+      $val = $oh->set( some_key => "some_value" );
+   }
+
    $val   = $oh->set( $key, $val );
    $val   = $oh->get( $key );
    $val   = $oh->delete( $key );              # del is an alias for delete
@@ -873,7 +905,87 @@ This document describes MCE::Shared::Ordhash version 1.808
    @vals  = $oh->sort( "key alpha" );         # by key $a cmp $b
    @vals  = $oh->sort( "key alpha desc" );    # by key $b cmp $a
 
-   # search capability key/val { =~ !~ eq ne lt le gt ge == != < <= > >= }
+   # included, sugar methods without having to call set/get explicitly
+
+   $len   = $oh->append( $key, $string );     #   $val .= $string
+   $val   = $oh->decr( $key );                # --$val
+   $val   = $oh->decrby( $key, $number );     #   $val -= $number
+   $val   = $oh->getdecr( $key );             #   $val--
+   $val   = $oh->getincr( $key );             #   $val++
+   $val   = $oh->incr( $key );                # ++$val
+   $val   = $oh->incrby( $key, $number );     #   $val += $number
+   $old   = $oh->getset( $key, $new );        #   $o = $v, $v = $n, $o
+
+   # pipeline, provides atomicity for shared objects, MCE::Shared v1.09+
+
+   @vals  = $oh->pipeline(                    # ( "a_a", "b_b", "c_c" )
+      [ "set", foo => "a_a" ],
+      [ "set", bar => "b_b" ],
+      [ "set", baz => "c_c" ],
+      [ "mget", qw/ foo bar baz / ]
+   );
+
+For normal hash behavior, the TIE interface is supported.
+
+   # non-shared or local construction for use by a single process
+
+   use MCE::Shared::Ordhash;
+
+   tie my %oh, "MCE::Shared::Ordhash", @pairs;
+   tie my %oh, "MCE::Shared::Ordhash";
+
+   # construction for sharing with other threads and processes
+   # the ordered option is needed to know to use MCE::Shared::Ordhash
+
+   use MCE::Shared;
+
+   tie my %oh, "MCE::Shared", { ordered => 1 }, @pairs;
+   tie my %oh, "MCE::Shared", ordered => 1;
+
+   # usage
+
+   my $val;
+
+   if ( !defined ( $val = $oh{some_key} ) ) {
+      $val = $oh{some_key} = "some_value";
+   }
+
+   $oh{some_key} = 0;
+
+   tied(%oh)->incrby("some_key", 20);
+   tied(%oh)->incrby(some_key => 20);
+
+=head1 SYNTAX for QUERY STRING
+
+Several methods take a query string for an argument. The format of the string
+is described below. In the context of sharing, the query mechanism is beneficial
+for the shared-manager process. It is able to perform the query where the data
+resides versus the client-process greping locally involving lots of IPC.
+
+   o Basic demonstration
+
+     @keys = $oh->keys( "query string given here" );
+     @keys = $oh->keys( "val =~ /pattern/" );
+
+   o Supported operators: =~ !~ eq ne lt le gt ge == != < <= > >=
+   o Multiple expressions delimited by :AND or :OR, mixed case allowed
+
+     "key eq 'some key' :or (val > 5 :and val < 9)"
+     "key eq some key :or (val > 5 :and val < 9)"
+     "key =~ /pattern/i :And val =~ /pattern/i"
+     "val eq foo baz :OR key !~ /pattern/i"
+
+     * key matches on keys in the hash
+     * likewise, val matches on values
+
+   o Quoting is optional inside the string
+
+     "key =~ /pattern/i :AND val eq 'foo bar'"   # val eq "foo bar"
+     "key =~ /pattern/i :AND val eq foo bar"     # val eq "foo bar"
+
+Examples.
+
+   # search capability key/val: =~ !~ eq ne lt le gt ge == != < <= > >=
    # key/val means to match against actual key/val respectively
 
    @keys  = $oh->keys( "key eq 'some key' :or (val > 5 :and val < 9)" );
@@ -891,72 +1003,20 @@ This document describes MCE::Shared::Ordhash version 1.808
    %pairs = $oh->pairs( "val >  $number" );
    %pairs = $oh->pairs( "val >= $number" );
 
-   @vals  = $oh->values( "key eq $string" );
-   @vals  = $oh->values( "key ne $string with space" );
-   @vals  = $oh->values( "key lt $string :or val =~ /$pat1|$pat2/" );
-   @vals  = $oh->values( "val le $string :and val eq 'foo bar'" );
-   @vals  = $oh->values( "val le $string :and val eq foo bar" );
-   @vals  = $oh->values( "val gt $string" );
-   @vals  = $oh->values( "val ge $string" );
-
-   # sugar methods without having to call set/get explicitly
-
-   $len   = $oh->append( $key, $string );     #   $val .= $string
-   $val   = $oh->decr( $key );                # --$val
-   $val   = $oh->decrby( $key, $number );     #   $val -= $number
-   $val   = $oh->getdecr( $key );             #   $val--
-   $val   = $oh->getincr( $key );             #   $val++
-   $val   = $oh->incr( $key );                # ++$val
-   $val   = $oh->incrby( $key, $number );     #   $val += $number
-   $old   = $oh->getset( $key, $new );        #   $o = $v, $v = $n, $o
-
-=head1 DESCRIPTION
-
-This module implements an ordered hash featuring tombstone deletion,
-inspired by the L<Hash::Ordered> module. An ordered hash means the key
-insertion order is preserved.
-
-It provides C<splice>, sorting, plus extra capabilities for use with
-L<MCE::Shared::Minidb>. Tombstone deletion is further optimized to not
-impact C<store>, C<push>, C<unshift>, and C<merge>. Tombstones are
-purged in-place for lesser memory consumption. In addition, C<pop> and
-C<shift> run optimally when an index is present. The optimization also
-applies to forward and reverse deletes.
-
-The end result is achieving a new level of performance, for a pure-Perl
-ordered hash implementation.
-
-=head1 SYNTAX for QUERY STRING
-
-Several methods in C<MCE::Shared::Ordhash> take a query string for an argument.
-
-   o Basic demonstration: @keys = $oh->keys( "val =~ /pattern/" );
-   o Supported operators: =~ !~ eq ne lt le gt ge == != < <= > >=
-   o Multiple expressions delimited by :AND or :OR
-   o Quoting optional inside the string
-
-     "key eq 'some key' :or (val > 5 :and val < 9)"
-     "key eq some key :or (val > 5 :and val < 9)"
-     "key =~ /pattern/i :and val =~ /pattern/i"
-     "key =~ /pattern/i :and val eq 'foo bar'"   # val eq "foo bar"
-     "key =~ /pattern/i :and val eq foo bar"     # val eq "foo bar"
-     "val eq foo baz :or key !~ /pattern/i"
-
-     * key matches on keys in the hash
-     * val matches on values
-
-=over 3
-
-=item * The modifiers C<:AND> and C<:OR> may be mixed case. e.g. C<:And>
-
-=back
+   @vals  = $oh->vals( "key eq $string" );
+   @vals  = $oh->vals( "key ne $string with space" );
+   @vals  = $oh->vals( "key lt $string :or val =~ /$pat1|$pat2/" );
+   @vals  = $oh->vals( "val le $string :and val eq 'foo bar'" );
+   @vals  = $oh->vals( "val le $string :and val eq foo bar" );
+   @vals  = $oh->vals( "val gt $string" );
+   @vals  = $oh->vals( "val ge $string" );
 
 =head1 API DOCUMENTATION
 
 This module involves TIE when accessing the object via hash-like behavior.
 Both non-shared and shared instances are impacted if doing so. Although likely
-fast enough for many use cases, use the OO interface if better performance is
-desired.
+fast enough for many use cases, the OO interface is recommended for best
+performance.
 
 =over 3
 
@@ -964,13 +1024,15 @@ desired.
 
 Constructs a new object, with an optional list of key-value pairs.
 
-   # non-shared
+   # non-shared or local construction for use by a single process
+
    use MCE::Shared::Ordhash;
 
    $oh = MCE::Shared::Ordhash->new( @pairs );
    $oh = MCE::Shared::Ordhash->new( );
 
-   # shared
+   # construction for sharing with other threads and processes
+
    use MCE::Shared;
 
    $oh = MCE::Shared->ordhash( @pairs );
@@ -1156,6 +1218,48 @@ described above. In scalar context, returns the size of the resulting list.
    @pairs = $oh->pairs( "val eq sun :OR val eq moon :OR val eq foo" );
    $len   = $oh->pairs( "key =~ /$pattern/" );
 
+=item pipeline ( [ func1, @args ], [ func2, @args ], ... )
+
+Combines multiple commands for the object to be processed serially. For shared
+objects, the call is made atomically due to single IPC to the shared-manager
+process. The C<pipeline> method is fully C<wantarray>-aware and receives a list
+of commands and their arguments. In scalar or list context, it returns data
+from the last command in the pipeline.
+
+   @vals = $oh->pipeline(                     # ( "a_a", "b_b", "c_c" )
+      [ "set", foo => "a_a" ],
+      [ "set", bar => "b_b" ],
+      [ "set", baz => "c_c" ],
+      [ "mget", qw/ foo bar baz / ]
+   );
+
+   $len = $oh->pipeline(                      # 3, same as $oh->len
+      [ "set", foo => "i_i" ],
+      [ "set", bar => "j_j" ],
+      [ "set", baz => "k_k" ],
+      [ "len" ]
+   );
+
+   $oh->pipeline(
+      [ "set", foo => "m_m" ],
+      [ "set", bar => "n_n" ],
+      [ "set", baz => "o_o" ]
+   );
+
+Current API available since 1.809.
+
+=item pipeline_ex ( [ func1, @args ], [ func2, @args ], ... )
+
+Same as C<pipeline>, but returns data for every command in the pipeline.
+
+   @vals = $oh->pipeline_ex(                  # ( "a_a", "b_b", "c_c" )
+      [ "set", foo => "a_a" ],
+      [ "set", bar => "b_b" ],
+      [ "set", baz => "c_c" ]
+   );
+
+Current API available since 1.809.
+
 =item pop
 
 Removes and returns the last key-value pair or value in scalar context of the
@@ -1276,7 +1380,10 @@ C<vals> is an alias for C<values>.
 =head1 SUGAR METHODS
 
 This module is equipped with sugar methods to not have to call C<set>
-and C<get> explicitly. The API resembles a subset of the Redis primitives
+and C<get> explicitly. In shared context, the benefit is atomicity and
+reduction in inter-process communication.
+
+The API resembles a subset of the Redis primitives
 L<http://redis.io/commands#strings> with key representing the hash key.
 
 =over 3
