@@ -12,7 +12,7 @@ use warnings;
 
 no warnings qw( threads recursion uninitialized numeric );
 
-our $VERSION = '1.808';
+our $VERSION = '1.809';
 
 ## no critic (Subroutines::ProhibitExplicitReturnUndef)
 
@@ -253,7 +253,8 @@ sub dequeue_nb {
       return;
    }
    else {
-      $_Q->{_nb_flag} = 1;
+      $_Q->{_nb_flag} = $_Q->_has_data() ? 1 : 0;
+
       return (defined $_cnt && $_cnt ne '1')
          ? $_Q->_dequeue($_cnt) : $_Q->_dequeue();
    }
@@ -611,18 +612,29 @@ MCE::Shared::Queue - Hybrid-queue helper class
 
 =head1 VERSION
 
-This document describes MCE::Shared::Queue version 1.808
+This document describes MCE::Shared::Queue version 1.809
+
+=head1 DESCRIPTION
+
+A queue helper class for use as a standalone or managed by L<MCE::Shared>.
+
+This module is mostly compatible with L<MCE::Queue> except for the C<gather>
+option which is not supported in this context. It provides a queue interface
+supporting normal and priority queues. Data from shared queues reside under
+the shared-manager process, otherwise locally.
 
 =head1 SYNOPSIS
 
-   # non-shared
+   # non-shared or local construction for use by a single process
+
    use MCE::Shared::Queue;
 
    my $qu = MCE::Shared::Queue->new(
       await => 1, fast => 0, queue => [ "." ]
    );
 
-   # shared
+   # construction for sharing with other threads and processes
+
    use MCE::Shared;
    use MCE::Shared::Queue;
 
@@ -632,7 +644,7 @@ This document describes MCE::Shared::Queue version 1.808
       fast   => 0
    );
 
-   # Possible values for "porder" and "type".
+   # possible values for "porder" and "type"
 
    porder =>
       $MCE::Shared::Queue::HIGHEST # Highest priority items dequeue first
@@ -644,7 +656,7 @@ This document describes MCE::Shared::Queue version 1.808
       $MCE::Shared::Queue::LILO    # Synonym for FIFO
       $MCE::Shared::Queue::FILO    # Synonym for LIFO
 
-   # Below, [ ... ] denotes optional parameters.
+   # below, [ ... ] denotes optional parameters
 
    $qu->await( [ $pending_threshold ] );
    $qu->clear();
@@ -665,17 +677,6 @@ This document describes MCE::Shared::Queue version 1.808
    $item  = $qu->peekp( $priority [, $index ] );
    @array = $qu->heap();
 
-=head1 DESCRIPTION
-
-Helper class for L<MCE::Shared>.
-
-This module is mostly compatible with L<MCE::Queue> except for the C<gather>
-option which is not supported in this context. It provides a queue interface
-supporting normal and priority queues.
-
-Data from shared queues reside under the shared-manager process, otherwise
-locally.
-
 =head1 API DOCUMENTATION
 
 =over 3
@@ -685,7 +686,8 @@ locally.
 Constructs a new object. Supported options are queue, porder, type, await, and
 fast.
 
-   # non-shared
+   # non-shared or local construction for use by a single process
+
    use MCE::Shared::Queue;
 
    $q1 = MCE::Shared::Queue->new();
@@ -700,7 +702,8 @@ fast.
    $q7 = MCE::Shared::Queue->new( await  => 1 );
    $q8 = MCE::Shared::Queue->new( fast   => 1 );
 
-   # shared;
+   # construction for sharing with other threads and processes
+
    use MCE::Shared;
    use MCE::Shared::Queue;
 
@@ -926,7 +929,6 @@ MCE::Shared::Queue supports both normal and priority queues.
 =item L<Thread::Queue>
 
 Thread::Queue is used as a template for identifying and documenting the methods.
-
 MCE::Shared::Queue is not fully compatible due to supporting normal and priority
 queues simultaneously; e.g.
 
@@ -942,17 +944,18 @@ queues simultaneously; e.g.
 
 =head1 LIMITATION
 
-Perl must have the L<IO::FDPass> module installed for constructing
-a shared C<queue> or C<condvar> while the shared-manager process is running.
-
-For platforms where C<IO::FDPass> is not feasible, construct C<queues> or
-C<condvars> first before other classes. The shared-manager process is delayed
-until sharing other classes or explicitly starting the process.
+Perl must have L<IO::FDPass> for constructing a shared C<condvar>, C<handle>,
+or C<queue> while the shared-manager process is running. For platforms where
+C<IO::FDPass> is not feasible, construct any C<condvar>, C<handle>, and
+C<queue> first before other classes. The shared-manager process is delayed
+until sharing other classes or starting the manager explicitly.
 
    use MCE::Shared;
 
-   my $q1 = MCE::Shared->queue();
-   my $cv = MCE::Shared->condvar();
+   my $cv  = MCE::Shared->condvar();
+   my $que = MCE::Shared->queue();
+
+   mce_open my $fh, ">>", "/path/to/file.log";
 
    MCE::Shared->start();
 
