@@ -12,7 +12,7 @@ use warnings;
 
 no warnings qw( threads recursion uninitialized numeric );
 
-our $VERSION = '1.808';
+our $VERSION = '1.809';
 
 ## no critic (TestingAndDebugging::ProhibitNoStrict)
 
@@ -100,26 +100,39 @@ MCE::Shared::Scalar - Scalar helper class
 
 =head1 VERSION
 
-This document describes MCE::Shared::Scalar version 1.808
+This document describes MCE::Shared::Scalar version 1.809
+
+=head1 DESCRIPTION
+
+A scalar helper class for use as a standalone or managed by L<MCE::Shared>.
 
 =head1 SYNOPSIS
 
-   # non-shared
+   # non-shared or local construction for use by a single process
+
    use MCE::Shared::Scalar;
 
    my $var = MCE::Shared::Scalar->new( $val );
 
-   # shared
+   # construction for sharing with other threads and processes
+
    use MCE::Shared;
 
    my $var = MCE::Shared->scalar( $val );
 
-   # oo interface
+   # scalar-like dereferencing
+
+   my $val = ${ $var };
+   ${ $var } = $val;
+
+   # OO interface
+
    $val = $var->set( $val );
    $val = $var->get();
    $len = $var->len();
 
-   # sugar methods without having to call set/get explicitly
+   # included, sugar methods without having to call set/get explicitly
+
    $val = $var->append( $string );     #   $val .= $string
    $val = $var->decr();                # --$val
    $val = $var->decrby( $number );     #   $val -= $number
@@ -129,15 +142,31 @@ This document describes MCE::Shared::Scalar version 1.808
    $val = $var->incrby( $number );     #   $val += $number
    $old = $var->getset( $new );        #   $o = $v, $v = $n, $o
 
-=head1 DESCRIPTION
+For normal scalar behavior, the TIE interface is supported.
 
-Helper class for L<MCE::Shared>.
+   # non-shared or local construction for use by a single process
+
+   use MCE::Shared::Scalar;
+
+   tie my $var, "MCE::Shared::Scalar";
+
+   # construction for sharing with other threads and processes
+
+   use MCE::Shared;
+
+   tie my $var, "MCE::Shared";
+
+   # usage
+
+   $var = 0;
+
+   tied($var)->incrby(20);
 
 =head1 API DOCUMENTATION
 
 This module may involve TIE when accessing the object via scalar dereferencing.
 Only shared instances are impacted if doing so. Although likely fast enough for
-many use cases, use the OO interface if better performance is desired.
+many use cases, the OO interface is recommended for best performance.
 
 =over 3
 
@@ -145,13 +174,15 @@ many use cases, use the OO interface if better performance is desired.
 
 Constructs a new object. Its value is undefined when C<value> is not specified.
 
-   # non-shared
+   # non-shared or local construction for use by a single process
+
    use MCE::Shared::Scalar;
 
    $var = MCE::Shared::Scalar->new( "foo" );
    $var = MCE::Shared::Scalar->new;
 
-   # shared
+   # construction for sharing with other threads and processes
+
    use MCE::Shared;
 
    $var = MCE::Shared->scalar( "bar" );
@@ -187,7 +218,10 @@ is not defined.
 =head1 SUGAR METHODS
 
 This module is equipped with sugar methods to not have to call C<set>
-and C<get> explicitly. The API resembles a subset of the Redis primitives
+and C<get> explicitly. In shared context, the benefit is atomicity and
+reduction in inter-process communication.
+
+The API resembles a subset of the Redis primitives
 L<http://redis.io/commands#strings> without the key argument.
 
 =over 3
