@@ -12,7 +12,7 @@ use warnings;
 
 no warnings qw( threads recursion uninitialized numeric );
 
-our $VERSION = '1.808';
+our $VERSION = '1.809';
 
 use Scalar::Util qw( looks_like_number );
 use MCE::Shared::Base;
@@ -120,6 +120,9 @@ sub next {
             if ( $step * ( $chunk_size - 1 ) + $seqn <= $endv ) {
                $seqe = $step * ( $chunk_size - 1 ) + $seqn;
             }
+            elsif ( $step == 1 ) {
+               $seqe = $endv;
+            }
             else {
                for my $i ( 1 .. $chunk_size ) {
                   last if ( $seqn > $endv );
@@ -132,6 +135,9 @@ sub next {
             if ( $step * ( $chunk_size - 1 ) + $seqn >= $endv ) {
                $seqe = $step * ( $chunk_size - 1 ) + $seqn;
             }
+            elsif ( $step == -1 ) {
+               $seqe = $endv;
+            }
             else {
                for my $i ( 1 .. $chunk_size ) {
                   last if ( $seqn < $endv );
@@ -208,11 +214,16 @@ MCE::Shared::Sequence - Sequence helper class
 
 =head1 VERSION
 
-This document describes MCE::Shared::Sequence version 1.808
+This document describes MCE::Shared::Sequence version 1.809
+
+=head1 DESCRIPTION
+
+A number sequence class for use as a standalone or managed by L<MCE::Shared>.
 
 =head1 SYNOPSIS
 
-   # non-shared
+   # non-shared or local construction for use by a single process
+
    use MCE::Shared::Sequence;
 
    my $seq_a = MCE::Shared::Sequence->new( $begin, $end, $step, $fmt );
@@ -222,8 +233,8 @@ This document describes MCE::Shared::Sequence version 1.808
       $begin, $end, $step, $fmt
    );
 
-   # shared
-   use MCE::Hobo;
+   # construction for sharing with other threads and processes
+
    use MCE::Shared;
 
    my $seq_a = MCE::Shared->sequence( 1, 100 );
@@ -233,6 +244,10 @@ This document describes MCE::Shared::Sequence version 1.808
       1, 100
    );
 
+   # example
+
+   use MCE::Hobo;
+
    sub parallel_a {
       my ( $id ) = @_;
       while ( defined ( my $num = $seq_a->next ) ) {
@@ -252,11 +267,9 @@ This document describes MCE::Shared::Sequence version 1.808
    MCE::Hobo->new( \&parallel_a, $_ ) for 1 .. 2;
    MCE::Hobo->new( \&parallel_b, $_ ) for 3 .. 4;
 
-   $_->join for MCE::Hobo->list();
+   # ... do other work ...
 
-=head1 DESCRIPTION
-
-A number sequence class for L<MCE::Shared>.
+   MCE::Hobo->waitall();
 
 =head1 API DOCUMENTATION
 
@@ -280,6 +293,8 @@ If C<bounds_only => 1> is specified, the C<next> method computes the C<begin>
 and C<end> values only for the chunk and not the numbers in between (hence
 boundaries only).
 
+   use MCE::Shared;
+
    # demo 1
    $seq1 = MCE::Shared->sequence(
       { chunk_size => 10, bounds_only => 0 },
@@ -314,7 +329,8 @@ boundaries only).
 
 Parameters may be given later with C<rewind> before calling C<next>.
 
-   # non-shared
+   # non-shared or local construction for use by a single process
+
    use MCE::Shared::Sequence;
 
    $seq = MCE::Shared::Sequence->new;
@@ -324,7 +340,8 @@ Parameters may be given later with C<rewind> before calling C<next>.
       { chunk_size => 10, bounds_only => 1 }, 1, 100
    );
 
-   # shared
+   # construction for sharing with other threads and processes
+
    use MCE::Shared;
 
    $seq = MCE::Shared->sequence;
