@@ -12,7 +12,7 @@ use warnings;
 
 no warnings qw( threads recursion uninitialized numeric once );
 
-our $VERSION = '1.808';
+our $VERSION = '1.809';
 
 ## no critic (BuiltinFunctions::ProhibitStringyEval)
 ## no critic (Subroutines::ProhibitExplicitReturnUndef)
@@ -1249,7 +1249,7 @@ sub _loop {
             $_Q->{_asem} = 0;
          }
 
-         $_Q->{_nb_flag} = 1;
+         $_Q->{_nb_flag} = $_Q->_has_data() ? 1 : 0;
 
          return;
       },
@@ -1417,6 +1417,7 @@ use constant {
 };
 
 my %_hash_deref_allow = (qw/
+   MCE::Shared::Cache   1
    MCE::Shared::Hash    1
    MCE::Shared::Ordhash 1
    Hash::Ordered        1
@@ -1796,6 +1797,7 @@ sub export {
    else {
       if    ( $_class eq 'MCE::Shared::Hash'    ) { $_data = $_item      }
       elsif ( $_class eq 'MCE::Shared::Ordhash' ) { $_data = $_item->[0] }
+      elsif ( $_class eq 'MCE::Shared::Cache'   ) { $_data = $_item->[0] }
       elsif ( $_class eq 'Hash::Ordered'        ) { $_data = $_item->[0] }
 
       ## no critic
@@ -1836,11 +1838,10 @@ sub iterator {
    }
 }
 
-# rewind ( begin, end, [ step, format ] )
-#
-# rewind ( index [, index, ... ] )
-# rewind ( key [, key, ... ] )
-# rewind ( "query string" )
+# rewind ( begin, end, [ step, format ] )  # Sequence
+# rewind ( index [, index, ... ] )         # Array
+# rewind ( key [, key, ... ] )             # Hash, Ordhash
+# rewind ( "query string" )                # Array, Hash, Ordhash
 # rewind ( )
 
 sub rewind {
@@ -2283,11 +2284,11 @@ MCE::Shared::Server - Server/Object packages for MCE::Shared
 
 =head1 VERSION
 
-This document describes MCE::Shared::Server version 1.808
+This document describes MCE::Shared::Server version 1.809
 
 =head1 DESCRIPTION
 
-Core class for L<MCE::Shared>. See documentation there.
+The core engine for L<MCE::Shared>. See documentation there.
 
 =head1 INDEX
 
